/*
 * Created on Mar 22, 2008
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.common;

import java.io.Serializable;

/**
 * This class contains the information on the time series plot. The
 * {@link org.cafesip.gwtcomp.client.ui.charting.LiveDataService} object can
 * display a line plot consisting of multiple series plotted against time. This
 * class represents the characteristics of one such a plot.
 * 
 * @author Amit Chatterjee
 * 
 */
public class TimeSeriesPlotInfo implements Serializable, PlotInfo
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = -5935471428769598805L;

    private SeriesInfo[] series;

    // Y-axis range

    private double minValue;

    private double maxValue;

    private long refreshInterval = 0L; // never refreshed by default

    private int numXTicks = 5;

    private int numYTicks = 5;

    private String yLabel = "Value";

    private int maxPlots = 20;

    /**
     * 
     * A constructor for this class.
     * 
     */
    public TimeSeriesPlotInfo()
    {
    }

    /**
     * A constructor for this class.
     * 
     * @param series
     *            properties for one or more series that is to be displayed
     * @param minValue
     *            bottom range. If all the series have a common bottom range, it
     *            must be specified here. It is also possible to specify
     *            separate bottom ranges for each the series.
     * @param maxValue
     *            top range. If all the series have a common top range, it must
     *            be specified here. It is also possible to specify separate top
     *            ranges for each the series.
     * @param refreshInterval
     *            specifies how frequently must the plot be refreshed. A value
     *            of 0 specifies no refresh after the inital data load.
     * 
     * @param numXTicks
     *            number of axis markers to display on the X axis.
     * @param numYTicks
     *            number of axis markers to display on the Y axis.
     * @param label
     *            Y Axis label.
     * @param maxPlots
     *            maximum number of plots to display for a given series.
     */
    public TimeSeriesPlotInfo(SeriesInfo[] series, double minValue,
            double maxValue, long refreshInterval, int numXTicks,
            int numYTicks, String label, int maxPlots)
    {
        this.series = series;
        this.minValue = minValue;
        this.maxValue = maxValue;
        this.refreshInterval = refreshInterval;
        this.numXTicks = numXTicks;
        this.numYTicks = numYTicks;
        yLabel = label;
        this.maxPlots = maxPlots;
    }

    /**
     * Similar to the constructor above but it uses the default values for some
     * of the parameters.
     */
    public TimeSeriesPlotInfo(SeriesInfo[] series, double minValue,
            double maxValue, long refreshInterval)
    {
        this.series = series;
        this.minValue = minValue;
        this.maxValue = maxValue;
        this.refreshInterval = refreshInterval;
    }

    /**
     * @return Returns the series.
     */
    public SeriesInfo[] getSeries()
    {
        return series;
    }

    /**
     * @param series
     *            The series to set.
     */
    public void setSeries(SeriesInfo[] series)
    {
        this.series = series;
    }

    /**
     * @return Returns the rangeBottom.
     */
    public double getMinValue()
    {
        return minValue;
    }

    /**
     * @param minValue
     *            The rangeBottom to set.
     */
    public void setMinValue(double minValue)
    {
        this.minValue = minValue;
    }

    /**
     * @return Returns the rangeTop.
     */
    public double getMaxValue()
    {
        return maxValue;
    }

    /**
     * @param maxValue
     *            The rangeTop to set.
     */
    public void setMaxValue(double maxValue)
    {
        this.maxValue = maxValue;
    }

    /**
     * @return Returns the refreshInterval.
     */
    public long getRefreshInterval()
    {
        return refreshInterval;
    }

    /**
     * @param refreshInterval
     *            The refreshInterval to set.
     */
    public void setRefreshInterval(long refreshInterval)
    {
        this.refreshInterval = refreshInterval;
    }

    /**
     * @return Returns the numXTicks.
     */
    public int getNumXTicks()
    {
        return numXTicks;
    }

    /**
     * @param numXTicks
     *            The numXTicks to set.
     */
    public void setNumXTicks(int numXTicks)
    {
        this.numXTicks = numXTicks;
    }

    /**
     * @return Returns the numYTicks.
     */
    public int getNumYTicks()
    {
        return numYTicks;
    }

    /**
     * @param numYTicks
     *            The numYTicks to set.
     */
    public void setNumYTicks(int numYTicks)
    {
        this.numYTicks = numYTicks;
    }

    /**
     * @return Returns the yLabel.
     */
    public String getYLabel()
    {
        return yLabel;
    }

    /**
     * @param label
     *            The yLabel to set.
     */
    public void setYLabel(String label)
    {
        yLabel = label;
    }

    /**
     * @return Returns the maxPlots.
     */
    public int getMaxPlots()
    {
        return maxPlots;
    }

    /**
     * @param maxPlots
     *            The maxPlots to set.
     */
    public void setMaxPlots(int maxPlots)
    {
        this.maxPlots = maxPlots;
    }
}
