/*
 * Created on Aug 11, 2007
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * A panel for data entry. The data entry panel allows common-styles to be applied
 * to screens from where the user submits data. For example, the data panel may
 * be used to create a "feedback form" where an user can enter his/her name, address,
 * etc. The panel allows you to add a title bar, a message bar for displaying error/warning/info
 * messages, a set of user input widgets (with label) and optionally, a set of buttons.
 * 
 * <p>
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/PersonalInfoPanel.java.html">PersonalInfoPanel.java</a></li>
 * <li><a href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/ImageUploadPanel.java.html">ImageUploadPanel.java</a></li>
 * </ul>
 * 
 * <p>
 * <u>CSS Style Rules:</u>
 * <ul class="css">
 * <li>.gwtcomp-DataEntryPanel { } - style for the panel itself</li>
 * <li>.gwtcomp-DataEntryPanelLabel { } - style for the label next to each input field</li>
 * <li>.gwtcomp-DataEntryPanelEntry { } - style for the input fields</li>
 * </ul>
 * <p>
 * 
 * <u>Screen-Shots:</u><br>
 * <img class='gallery' src='../../../../../../screen-shots/data_entry_panel.gif'/>
 * 
 * 
 * @author Amit Chatterjee
 * 
 */
public class DataEntryPanel extends SimplePanel
{
    private FlexTable table;

    private TitleBar titleBar = null;

    private MessageBar messageBar = null;

    private ButtonPanel buttonBar;

    /**
     *  A constructor for this class.
     * 
     */
    public DataEntryPanel()
    {
        this (null, null);
    }
    
   
    /**
     *  A constructor for this class.
     * 
     *  @param titleBar a title bar object that is displayed
     *  @param messageBar a message bar object that is displayed
     */
    public DataEntryPanel(TitleBar titleBar, MessageBar messageBar)
    {
        super();
        setStyleName("gwtcomp-DataEntryPanel");

        table = new FlexTable();
        table.setWidth("100%");
        table.setCellPadding(5);
        table.setCellSpacing(5);
        table.setWidth("100%");
        setWidget(table);
        
        buttonBar = new ButtonPanel();
        
        table.insertRow(0);
        table.getFlexCellFormatter().setColSpan(0, 0, 2);
        table.setWidget(0, 0, buttonBar);
        buttonBar.setWidth("100%"); 
        
        if (titleBar != null)
        {
            setTitleBar(titleBar);
        }
        
        if (messageBar != null)
        {
            setMessageBar(messageBar);
        }
    }
    
    

    /**
     * Returns the button bar at the bottom of the panel. You can get this object
     * and add additional buttons to the button bar.
     * 
     * @return Returns the button bar.
     */
    public ButtonPanel getButtonBar()
    {
        return buttonBar;
    }

    /**
     * Returns the message bar at the top of the panel. You can set messages on the 
     * message bar after getting the object.
     * 
     * @return Returns the message bar - null if the bar has not been set.
     */
    public MessageBar getMessageBar()
    {
        return messageBar;
    }


    /**
     * Sets a message bar.
     * @param messageBar The message bar to set.
     */
    public void setMessageBar(MessageBar messageBar)
    {
        this.messageBar = messageBar;
        
        int row = 0;
        if (titleBar != null)
        {
            row = 1;
        }
        
        table.insertRow(row);
        table.getFlexCellFormatter().setColSpan(row, 0, 2);
        table.setWidget(row, 0, messageBar);
        messageBar.setWidth("100%");        
    }

    
    /**
     * Returns the table that is used for laying out the data entry panel.
     * 
     * @return Returns the table.
     */
    public FlexTable getTable()
    {
        return table;
    }

    /**
     * Returns the title bar at the top of the panel. You can set title text and help URL on the 
     * title bar after getting the object.
     * 
     * @return Returns the titleBar.
     */
    public TitleBar getTitleBar()
    {
        return titleBar;
    }

    /**
     * Sets a title bar.
     * @param titleBar The title bar to set.
     */
    public void setTitleBar(TitleBar titleBar)
    {
        this.titleBar = titleBar;
                
        table.insertRow(0);
        table.getFlexCellFormatter().setColSpan(0, 0, 2);
        table.setWidget(0, 0, titleBar);
        titleBar.setWidth("100%");
    }
    
    /**
     * Add a button to the button bar
     * 
     * @param button button to add.
     */
    public void addButton(Button button)
    {
        buttonBar.addButton(button);
    }
    
    /**
     * Add a label and an input field.
     * 
     * @param label label for the input field.
     * @param entry the input field.
     * @param required true if the parameter entry is mandatory.
     */
    public void addField(Label label, Widget entry, boolean required)
    {
        int row = table.getRowCount();
        table.insertRow(row-1);
     
        if (required == true)
        {
            label.setText(label.getText() + " (*)");
        }
        
        label.setWordWrap(false);
        table.getCellFormatter().setWordWrap(row-1, 0, false);
        table.setWidget(row-1, 0, label);
        table.getCellFormatter().setStyleName(row-1, 0, "gwtcomp-DataEntryPanelLabel");
        table.setWidget(row-1, 1, entry);
        table.getCellFormatter().setStyleName(row-1, 1, "gwtcomp-DataEntryPanelEntry");
    }
    
    /**
     * Same as the other addField() method - mandatory flag is set to false automatically.
     * 
     * @param label
     * @param entry
     */
    public void addField(Label label, Widget entry)
    {
        addField(label, entry, false);
    }
    
    /**
     * Adds a single widget to the form layout instead of adding a label/entry.
     * 
     * @param entry the widget that is to be added.
     */
    public void addField(Widget entry)
    {
        int row = table.getRowCount();
        table.insertRow(row-1);
     
        table.getFlexCellFormatter().setColSpan(row-1, 0, 2);
        table.setWidget(row-1, 0, entry);
        table.getCellFormatter().setStyleName(row-1, 1, "gwtcomp-DataEntryPanelEntry");
    }    
}
