/*
 * Created on Aug 30, 2006
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import java.util.HashMap;

import org.cafesip.gwtcomp.client.helpers.DraggableElement;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.Widget;

/**
 * An utility class for making a widget "draggable". A draggable widget can be
 * dragged and dropped into a
 * {@link org.cafesip.gwtcomp.client.ui.DroppableList} object. Each draggable
 * object must have a unique id and a "class" associated with it. The class (not
 * to be confused with a class in object-oriented programming) is used to
 * classify the object. For example, a draggable object with id="123" and
 * class="country" can be dropped into a DroppableList that accepts objects of
 * class="country". Therefore, it is possible to have more than one
 * DroppableList in a screen that accepts objects of different class. Note that
 * the id attribute is the same as the DOM id and must be unique for the
 * underlying HTML document.
 * <p>
 * Important note: the makeDraggable() method must be invoked only after the
 * object has been attached to the DOM document because of the limitation of the
 * underlying implementation. If the application requires a label that is
 * draggable, use the {@link org.cafesip.gwtcomp.client.ui.DraggableLabel}
 * instead. The makeDraggable() method does not have to be invoked for this
 * class.
 * <p>
 * The draggable/droppable classes use the scriptaculous javascript library for
 * providing the functionality. Therefore, you must add the following segment to
 * your application HTML file (see the GWTCompExamples.html for an example).
 * 
 * <pre>
 *       &lt;script src=&quot;gwtcomp-scripts/scriptaculous/prototype.js&quot; type=&quot;text/javascript&quot;&gt;&lt;/script&gt;
 *       &lt;script src=&quot;gwtcomp-scripts/scriptaculous/scriptaculous.js&quot; type=&quot;text/javascript&quot;&gt;&lt;/script&gt;
 *       &lt;script src=&quot;gwtcomp-scripts/scriptaculous/dragdrop.js&quot; type=&quot;text/javascript&quot;&gt;&lt;/script&gt;
 * </pre>
 * 
 * <p>
 * 
 * @author Amit Chatterjee
 * 
 */
public class Draggable
{
    private Element draggable;

    private Widget object;

    private String clazz;

    private String name;

    private String id;

    private static HashMap<String, DraggableElement> draggables = new HashMap<String, DraggableElement>();

    private Draggable(Widget object, String name, String id, String clazz)
    {
        this.id = id;
        this.object = object;
        this.clazz = clazz;
        this.name = name;

        Element e = object.getElement();
        DOM.setElementAttribute(e, "id", id);

        init(e, id, clazz);
        draggables.put(id, new DraggableElement(name, clazz));
    }

    private native void init(Element e, String id, String clazz) /*-{
        this.@org.cafesip.gwtcomp.client.ui.Draggable::draggable = 
        new $wnd.Draggable(id, {revert: true});
        }-*/;

    private native void destroy() /*-{
        var e = this.@org.cafesip.gwtcomp.client.ui.Draggable::draggable;
        e.destroy();
        }-*/;

    /**
     * This method <b>must</b> be called when the object is no longer attached
     * with the DOM document. Otherwise, the object will not be removed from the
     * draggables list and will cause memory leak and potentially other issues.
     */
    public void dispose()
    {
        draggables.remove(id);
        destroy();
    }

    /**
     * Returns the draggable widget.
     * 
     * @return Returns the object.
     */
    public Widget getObject()
    {
        return object;
    }

    /**
     * Returns the class.
     * 
     * @return Returns the clazz.
     */
    public String getClazz()
    {
        return clazz;
    }

    /**
     * Returns the DOM element associated with the draggable object.
     * 
     * @return Returns the draggable.
     */
    public Element getDraggable()
    {
        return draggable;
    }

    /**
     * Returns the name of the draggable object.
     * 
     * @return Returns the name.
     */
    public String getName()
    {
        return name;
    }

    /**
     * Returns a list of draggable items. The key for the HashMap is the id and
     * the value is an object of the type DraggableElement.
     * 
     * @return Returns the draggables.
     */
    public static HashMap<String, DraggableElement> getDraggables()
    {
        return draggables;
    }

    /**
     * This static method is used to make a widget draggable. The dispose()
     * method must be called to release the draggable nature of the object.
     * 
     * @param object
     *            widget to make draggable.
     * @param name
     *            the name of the object. The DroppableList object identifies
     *            the object with this name.
     * @param id
     *            the DOM id of the object.
     * @param clazz
     *            the class of the widget.
     * 
     * @return a draggable object.
     */
    public static Draggable makeDraggable(Widget object, String name,
            String id, String clazz)
    {
        return new Draggable(object, name, id, clazz);
    }
}
