/*
 * Created on Sep 1, 2006
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import java.util.ArrayList;

import org.cafesip.gwtcomp.client.helpers.DraggableElement;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasName;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * A list object where a draggable item can be dropped into. As explained in the
 * documentation for {@link org.cafesip.gwtcomp.client.ui.Draggable}, any
 * visual widget can be converted into a draggable object which can be dragged
 * by the user and dropped to this object. Alternatively, applications can use
 * the {@link org.cafesip.gwtcomp.client.ui.DraggableLabel}. In addition, the
 * users can delete any item in the list.This object implements the GWT HasName
 * interface. Therefore it can be used with a GWT FormPanel. When used with a
 * FormPanel, it behaves like a ListBox with all its items selected.
 * <p>
 * The draggable/droppable classes use the scriptaculous javascript library for
 * providing the functionality. Therefore, you must add the following segment to
 * your application HTML file (see the GWTCompExamples.html for an example).
 * 
 * <pre>
 *       &lt;script src=&quot;gwtcomp-scripts/scriptaculous/prototype.js&quot; type=&quot;text/javascript&quot;&gt;&lt;/script&gt;
 *       &lt;script src=&quot;gwtcomp-scripts/scriptaculous/scriptaculous.js&quot; type=&quot;text/javascript&quot;&gt;&lt;/script&gt;
 *       &lt;script src=&quot;gwtcomp-scripts/scriptaculous/dragdrop.js&quot; type=&quot;text/javascript&quot;&gt;&lt;/script&gt;
 * </pre>
 * 
 * <p>
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a
 * href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/DraggableSample.java.html">DraggableSample.java</a></li>
 * </ul>
 * 
 * <p>
 * <u>CSS Style Rules:</u>
 * <ul class="css">
 * <li>.gwtcomp-DroppableList { } - style for the list itself</li>
 * <li>.gwtcomp-DroppableList-DroppedItem { } - style for dropped items</li>
 * <li>.gwtcomp-DroppableList-DroppedItem-DeleteIcon { } - style for delete
 * icon next to a dropped item</li>
 * </ul>
 * 
 * <p>
 * <u>Screen-Shots:</u><br>
 * <img class='gallery' title="A list with draggable lables to the left."
 * src='../../../../../../screen-shots/droppable_list_1.gif'/>
 * <p>
 * <u>Screen-Shots:</u><br>
 * <img class='gallery' title="The list icon itself"
 * src='../../../../../../screen-shots/droppable_list_2.gif'/>
 * <p>
 * 
 * @author Amit Chatterjee
 * 
 */
public class DroppableList extends Composite implements HasName
{
    private static final String BASKET_FULL_ICON = GWT.getModuleBaseURL()
            + "gwtcomp-icons/trash_full.png";

    private static final String BASKET_EMPTY_ICON = GWT.getModuleBaseURL()
            + "gwtcomp-icons/trash_empty.png";

    private String id;

    private String clazz;

    private VerticalPanel panel;

    private ListBox list;

    private Image basket;

    private ArrayList<DroppableListListener> listeners = new ArrayList<DroppableListListener>();

    private String basketEmptyIcon = BASKET_EMPTY_ICON;

    private String basketFullIcon = BASKET_FULL_ICON;

    /**
     * A constructor for this class.
     * 
     * @param id
     *            DOM id of the list.
     * @param clazz
     *            the class of items it accepts. See
     *            {@link org.cafesip.gwtcomp.client.ui.Draggable} for details.
     */
    public DroppableList(String id, String clazz)
    {
        this.clazz = clazz;
        this.id = id;

        panel = new VerticalPanel();
        initWidget(panel);
        panel.setSpacing(5);

        panel.setWidth("100%");
        panel.setStyleName("gwtcomp-DroppableList");

        list = new ListBox(true);
        panel.add(list);
        list.setVisible(false);

        basket = new Image(basketEmptyIcon);
        panel.add(basket);
        panel.setCellHorizontalAlignment(basket,
                HasHorizontalAlignment.ALIGN_CENTER);

        Element e = basket.getElement();
        DOM.setElementAttribute(e, "id", id);
    }

    @SuppressWarnings("unused")
    private void onDrop(String id)
    {
        DraggableElement e = Draggable.getDraggables().get(
                id);
        if (e == null)
        {
            return;
        }

        if (!e.getClazz().equals(clazz))
        {
            Audio.getInstance().playWarning();
            return;
        }

        String name = e.getName();
        int count = list.getItemCount();
        for (int i = 0; i < count; i++)
        {
            if (name.equals(list.getItemText(i)) == true)
            {
                return;
            }
        }

        addItem(name, true);
    }

    /**
     * Add an item to the list.
     * 
     * @param name
     *            the name of the item.
     */
    public void addItem(String name)
    {
        addItem(name, false);
    }

    /**
     * Adds an items to the list and invokes listeners if registered.
     * 
     * @param name
     *            name of the item.
     * @param fireEvents
     *            listeners are invoked if set to true.
     */
    public void addItem(String name, boolean fireEvents)
    {
        int index = list.getItemCount();
        list.addItem(name);
        list.setSelectedIndex(index);
        basket.setUrl(basketFullIcon);

        HorizontalPanel hp = new HorizontalPanel();
        panel.add(hp);
        hp.setStyleName("gwtcomp-DroppableList-DroppedItem");
        hp.setWidth("100%");

        HTML hname = new HTML(name);
        hp.add(hname);
        hp.setCellHorizontalAlignment(hp, HasHorizontalAlignment.ALIGN_LEFT);

        Image im = new Image(GWT.getModuleBaseURL()
                + "gwtcomp-icons/cancel.png");
        hp.add(im);
        im.addStyleName("gwtcomp-DroppableList-DroppedItem-DeleteIcon");
        hp.setCellHorizontalAlignment(im, HasHorizontalAlignment.ALIGN_RIGHT);

        class DeleteItemListener implements ClickListener
        {
            public String name;

            public DeleteItemListener(String name)
            {
                this.name = name;
            }

            public void onClick(Widget sender)
            {
                deleteItem(name);
                for (int i = 0; i < listeners.size(); i++)
                {
                    DroppableListListener listener = listeners.get(i);
                    listener.itemRemove(name);
                }
            }
        }

        im.addClickListener(new DeleteItemListener(name));

        if (fireEvents)
        {
            for (int i = 0; i < listeners.size(); i++)
            {
                DroppableListListener listener = listeners.get(i);
                listener.itemAdded(name);
            }
        }
    }

    /**
     * Delete an item from the list.
     * 
     * @param name
     *            name of the item.
     */
    public void deleteItem(String name)
    {
        int count = list.getItemCount();
        for (int i = 0; i < count; i++)
        {
            String lname = list.getItemText(i);
            if (lname.equals(name))
            {
                removeItemFromList(count, i);
                return;
            }
        }
    }

    private void removeItemFromList(int count, int i)
    {
        list.removeItem(i);
        panel.remove(i + 2);

        if (count == 1)
        {
            // this is the last item
            basket.setUrl(basketEmptyIcon);
        }
    }

    /**
     * Get the number of items in the list.
     * 
     * @return item count.
     */
    public int getItemCount()
    {
        return list.getItemCount();
    }

    /**
     * Get the item at a specified index.
     * 
     * @param index
     *            index
     * @return the name of the item.
     */
    public String getItem(int index)
    {
        return list.getItemText(index);
    }

    /**
     * Deletes an item at the specified index.
     * 
     * @param index
     */
    public void deleteItem(int index)
    {
        removeItemFromList(getItemCount(), index);
    }

    /**
     * Gets all the items in the list.
     * 
     * @return the items
     */
    public String[] getItems()
    {
        String[] items = new String[getItemCount()];
        for (int i = 0; i < items.length; i++)
        {
            items[i] = getItem(i);
        }

        return items;
    }

    private native void makeDroppable() /*-{
                 var obj = this;
                 var id = this.@org.cafesip.gwtcomp.client.ui.DroppableList::id;
                 var clazz = this.@org.cafesip.gwtcomp.client.ui.DroppableList::clazz;
                 $wnd.Droppables.add(id, 
                 { onDrop: function(element) 
                 {
                 obj.@org.cafesip.gwtcomp.client.ui.DroppableList::onDrop(Ljava/lang/String;)(element.id);
                 }
                 });
                 }-*/;

    /*
     * @see com.google.gwt.user.client.ui.Widget#onLoad()
     */
    protected void onLoad()
    {
        makeDroppable();
    }

    /**
     * @see com.google.gwt.user.client.ui.HasName#getName()
     */
    public String getName()
    {
        return list.getName();
    }

    /**
     * @see com.google.gwt.user.client.ui.HasName#setName(java.lang.String)
     */
    public void setName(String name)
    {
        list.setName(name);
    }

    /**
     * Adds a listener for listening to drop events.
     * 
     * @param listener
     */
    public void addListener(DroppableListListener listener)
    {
        listeners.add(listener);
    }

    /**
     * Removes a listener.
     * 
     * @param listener
     */
    public void removeListener(DroppableListListener listener)
    {
        listeners.remove(listener);
    }

    /**
     * Returns the currently-set icon that is displayed when the list is empty.
     * 
     * @return Returns the URL for the basketEmptyIcon.
     */
    public String getBasketEmptyIcon()
    {
        return basketEmptyIcon;
    }

    /**
     * Sets the URL for the icon that is displayed when the list is empty.
     * 
     * @param basketEmptyIcon
     *            The URL for basketEmptyIcon to set.
     */
    public void setBasketEmptyIcon(String basketEmptyIcon)
    {
        if (basket.getUrl().equals(this.basketEmptyIcon))
        {
            basket.setUrl(basketEmptyIcon);
        }
        this.basketEmptyIcon = basketEmptyIcon;
    }

    /**
     * Returns the currently-set icon that is displayed when the list has items.
     * 
     * @return Returns the URL for the basketFullIcon.
     */
    public String getBasketFullIcon()
    {
        return basketFullIcon;
    }

    /**
     * Sets the URL for the icon that is displayed when the list has items.
     * 
     * @param basketFullIcon
     *            The URL for the basketFullIcon to set.
     */
    public void setBasketFullIcon(String basketFullIcon)
    {
        if (basket.getUrl().equals(this.basketFullIcon))
        {
            basket.setUrl(basketFullIcon);
        }
        this.basketFullIcon = basketFullIcon;
    }

    /**
     * @return Returns the id.
     */
    public String getId()
    {
        return id;
    }

    /**
     * @return Returns the clazz.
     */
    public String getClazz()
    {
        return clazz;
    }
}
