/**
 * Created on Feb 10, 2008
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *   http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.cafesip.gwtcomp.client.ui;

import java.util.ArrayList;
import java.util.Iterator;

import org.cafesip.gwtcomp.client.utils.HTMLHelper;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;

/**
 * A panel for an editable list that allows the user to add elements to a list
 * and remove elements from the list. This composite consists of the following
 * widgets:
 * <ul>
 * <li>a TextBox for entering a new element to be added to the list</li>
 * <li>a Button that when clicked adds the TextBox contents to the list</li>
 * <li>a ListBox containing the current list of elements</li>
 * <li>a Button that when clicked removes the selected list element(s) from the
 * list.</li>
 * </ul>
 * You may use your own widget(s) rather than the default widgets created by
 * this component by calling the appropriate constructor.
 * 
 * When list elements are added or removed by the user, the action taken by this
 * widget can be configured as follows.
 * <ol>
 * <li>[DEFAULT] Perform no validation on the user input/action - this will
 * cause elements to be added or removed as directed by the user without any
 * validation checks.</li>
 * <li>Perform validation, as described below, on user input/action before
 * modifying the list. After instantiating this object, you must call
 * setMessageBar(MessageBar) to cause validation to happen. You may optionally
 * call setErrorMessagePrefix() to override the default text that precedes the
 * error message(s) output when validation fails.</li>
 * <li>Invoke your own logic when the user clicks the Add or Remove Buttons.
 * After instantiating this object, you must call
 * setAddButtonListener(ClickListener) or setRemoveButtonListener(ClickListener)
 * or both. In this case, none of the logic contained in this widget will be
 * executed when the button is clicked.</li>
 * </ol>
 * 
 * When validation is done by this widget, the default behavior is as follows.
 * <ul>
 * <li>When the Add Button is clicked by the user, this widget disallows
 * duplicate list elements.</li>
 * <li>When the Remove Button is clicked by the user, this widget prevents the
 * user from removing all of the elements from the list.</li>
 * </ul>
 * 
 * The default validation behavior can be changed by calling the following
 * methods: allowDuplicates(true), allowEmptyList(true).
 * 
 * <p>
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a
 * href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/EditableListSample.java.html">EditableListSample.java</a></li>
 * </ul>
 * 
 * <p>
 * <u>CSS Style Rules:</u>
 * <ul class="css">
 * <li>.gwtcomp-EditableList { } - style for the panel itself</li>
 * <li>.gwtcomp-EditableListNewElement { } - style for the new list element
 * text box </li>
 * <li>.gwtcomp-EditableListAddButton { } - style for the Add new list element
 * button </li>
 * <li>.gwtcomp-EditableListContents { } - style for the list widget </li>
 * <li>.gwtcomp-EditableListRemoveButton { } - style for the Remove element(s)
 * button </li>
 * </ul>
 * <p>
 * 
 * <u>Screen-Shots:</u><br>
 * <img class='gallery' src='../../../../../../screen-shots/editable_list.gif'/>
 * <p>
 * In the above screen-shot, the top row contains the text box for the user to
 * enter a new element to be added to the list and the Add button that when
 * clicked adds the text box contents to the list. The second row contains the
 * list of elements itself and the Remove button. The user may select one or
 * more elements from the list and click the Remove button which causes the
 * selected element(s) to be removed from the list.
 * 
 * @author Becky McElroy
 * 
 */
public class EditableList extends Composite implements ClickListener,
        KeyboardListener
{
    private TextBox newElement;

    private Button addButton;

    private ListBox list;

    private Button removeButton;

    private MessageBar messageBar;

    private String errorMessagePrefix = "We encountered the following errors while processing your request.";

    private boolean allowDuplicates;

    private boolean allowEmptyList;

    /**
     * An empty constructor for this class. If you use this constructor, you
     * will get the default widgets as created by this class. Please read the
     * class-level javadoc for information on what additional methods to call,
     * if any, after instantiating this object depending on the validation
     * action or behavior you want.
     */
    public EditableList()
    {
        this(null, null, null, null);
    }

    /**
     * A constructor for this class. This constructor allows you to define the
     * characteristics or appearance of one or more of the widgets used by this
     * composite widget. For example, by providing the addButton yourself, you
     * can control the label text for the button, the image used on the button
     * and the tooltip for the button.
     * 
     * Please read the class-level javadoc for information on what additional
     * methods to call, if any, after instantiating this object depending on the
     * validation action or behavior you want.
     * 
     * @param newElement
     *            the TextBox for entering a new element to be added to the list
     * @param addButton
     *            the Button that when clicked adds the TextBox contents to the
     *            list
     * @param list
     *            the ListBox containing the current list of elements
     * @param removeButton
     *            the Button that when clicked removes the selected list
     *            element(s) from the list
     */
    public EditableList(TextBox newElement, Button addButton, ListBox list,
            Button removeButton)
    {
        super();

        FlowPanel panel = new FlowPanel();
        initWidget(panel);
        panel.setStyleName("gwtcomp-EditableList");

        if (newElement == null)
        {
            newElement = new TextBox();
            newElement.setText("");
            newElement.setVisibleLength(40);
        }
        newElement.setStyleName("gwtcomp-EditableListNewElement");
        newElement.addKeyboardListener(this);
        this.newElement = newElement;

        if (addButton == null)
        {
            addButton = new Button();
            addButton.setHTML(HTMLHelper.imageWithText(GWT.getModuleBaseURL()
                    + "gwtcomp-icons/edit_add.png", "Add"));
            addButton.setTitle("Add element to list");
        }
        addButton.setStyleName("gwtcomp-EditableListAddButton");
        addButton.addClickListener(this);
        this.addButton = addButton;

        if (list == null)
        {
            list = new ListBox();
            list.setMultipleSelect(true);
            list.setVisibleItemCount(8);
        }
        list.setStyleName("gwtcomp-EditableListContents");
        this.list = list;

        if (removeButton == null)
        {
            removeButton = new Button();
            removeButton.setHTML(HTMLHelper.imageWithText(GWT
                    .getModuleBaseURL()
                    + "gwtcomp-icons/editdelete.png", "Remove"));
            removeButton.setTitle("Remove selected element(s) from list");
        }
        removeButton.setStyleName("gwtcomp-EditableListRemoveButton");
        removeButton.addClickListener(this);
        this.removeButton = removeButton;

        panel.add(newElement);
        panel.add(addButton);
        panel.add(list);
        panel.add(removeButton);
    }

    public void onClick(Widget sender)
    {
        if (sender == addButton)
        {
            addElement();
            return;
        }

        if (sender == removeButton)
        {
            removeSelectedElements();
            return;
        }
    }

    public void onKeyDown(Widget sender, char keyCode, int modifiers)
    {
    }

    public void onKeyPress(Widget sender, char keyCode, int modifiers)
    {
    }

    public void onKeyUp(Widget sender, char keyCode, int modifiers)
    {
        if (keyCode == KeyboardListener.KEY_ENTER)
        {
            addButton.click();
        }
    }

    /**
     * Call this method to invoke your own logic when the user clicks the Add
     * button.
     * 
     * @param listener
     */
    public void setAddButtonListener(ClickListener listener)
    {
        addButton.removeClickListener(this);
        addButton.addClickListener(listener);
    }

    /**
     * Call this method to invoke your own logic when the user clicks the Remove
     * button.
     * 
     * @param listener
     */
    public void setRemoveButtonListener(ClickListener listener)
    {
        removeButton.removeClickListener(this);
        removeButton.addClickListener(listener);
    }

    private void addElement()
    {
        String element = newElement.getText().trim();

        if (element.length() == 0)
        {
            return;
        }

        if (messageBar != null)
        {
            ArrayList<String> errors = validateAdd(element);

            if (errors.size() > 0)
            {
                outputErrorMessage(errors);
                return;
            }

            messageBar.clearMessage();
        }

        list.addItem(element);
        newElement.setText("");
    }

    private void outputErrorMessage(ArrayList<String> errors)
    {
        String[] messages = new String[errors.size()];
        errors.toArray(messages);

        messageBar.setMessage(errorMessagePrefix
                + HTMLHelper.orderedList(messages), MessageBar.SEVERITY_ERROR);
    }

    private void removeSelectedElements()
    {
        ArrayList<Integer> selected = new ArrayList<Integer>();
        for (int i = list.getItemCount() - 1; i >= 0; i--)
        {
            if (list.isItemSelected(i))
            {
                selected.add(new Integer(i));
            }
        }

        if (messageBar != null)
        {
            ArrayList<String> errors = validateRemove(selected);

            if (errors.size() > 0)
            {
                outputErrorMessage(errors);
                return;
            }

            messageBar.clearMessage();
        }

        Iterator<Integer> i = selected.iterator();
        while (i.hasNext())
        {
            int index = i.next().intValue();
            list.removeItem(index);
        }
    }

    private ArrayList<String> validateAdd(String element)
    {
        ArrayList<String> errors = new ArrayList<String>();

        if (allowDuplicates == false)
        {
            if (listContains(element))
            {
                errors.add("The element being added is already in the list");
            }
        }

        return errors;
    }

    private ArrayList<String> validateRemove(ArrayList<Integer> selected)
    {
        ArrayList<String> errors = new ArrayList<String>();

        if (allowEmptyList == false)
        {
            if ((selected.size() == list.getItemCount())
                    && (list.getItemCount() > 0))
            {
                errors.add("All of the elements in the list cannot be removed");
            }
        }

        return errors;
    }

    /**
     * Call this method to find out if the list contains the given element.
     * 
     * @param element
     *            the element of interest
     * @return true if the element is in the list, false otherwise.
     */
    public boolean listContains(String element)
    {
        int size = list.getItemCount();
        for (int i = 0; i < size; i++)
        {
            if (list.getItemText(i).equals(element))
            {
                return true;
            }
        }

        return false;
    }

    /**
     * Set the contents of the ListBox.
     * 
     * @param elements
     *            ArrayList of String to set the list contents to.
     */
    public void setList(ArrayList<String> elements)
    {
        list.clear();
        Iterator<String> i = elements.iterator();
        while (i.hasNext())
        {
            list.addItem(i.next());
        }
    }

    /**
     * Returns the contents of the ListBox.
     * 
     * @return a copy of the list contents in the form of ArrayList of String.
     */
    public ArrayList<String> getList()
    {
        ArrayList<String> elements = new ArrayList<String>();
        int size = list.getItemCount();
        for (int i = 0; i < size; i++)
        {
            elements.add(list.getItemText(i));
        }

        return elements;
    }

    /**
     * Returns the TextBox widget where the user types in a new element to add
     * to the list.
     * 
     * @return Returns the newElement TextBox.
     */
    public TextBox getNewElement()
    {
        return newElement;
    }

    /**
     * Returns the MessageBar as set by a previous call to setMessageBar() or
     * null if setMessageBar() has not been called.
     * 
     * @return Returns the messageBar.
     */
    public MessageBar getMessageBar()
    {
        return messageBar;
    }

    /**
     * Sets the MessageBar for use by this widget's validation. Setting the
     * MessageBar to a non-null value causes this widget to perform validation
     * when the user hits the Add or Remove button.
     * 
     * @param messageBar
     *            The messageBar to set.
     */
    public void setMessageBar(MessageBar messageBar)
    {
        this.messageBar = messageBar;
    }

    /**
     * Returns the error message prefix string that is output along with
     * validation error(s).
     * 
     * @return Returns the errorMessagePrefix.
     */
    public String getErrorMessagePrefix()
    {
        return errorMessagePrefix;
    }

    /**
     * Sets the error message prefix string to be output along with validation
     * error(s) when this widget performs Add/Remove button validation.
     * 
     * @param errorMessagePrefix
     *            The errorMessagePrefix to set.
     */
    public void setErrorMessagePrefix(String errorMessagePrefix)
    {
        this.errorMessagePrefix = errorMessagePrefix;
    }

    /**
     * Returns whether or not validation when done by this widget would allow
     * duplicate entries in the list.
     * 
     * @return true if setAllowDuplicates(true) has previously been called,
     *         false otherwise.
     */
    public boolean duplicatesAllowed()
    {
        return allowDuplicates;
    }

    /**
     * Sets whether or not validation when done by this widget should allow
     * duplicate entries in the list.
     * 
     * @param allowDuplicates
     *            true if duplicate elements in the list should be allowed,
     *            false otherwise.
     */
    public void allowDuplicates(boolean allowDuplicates)
    {
        this.allowDuplicates = allowDuplicates;
    }

    /**
     * Returns whether or not validation when done by this widget would allow
     * the user to remove all of the elements from the list.
     * 
     * @return true if setAllowEmptyList(true) has previously been called, false
     *         otherwise.
     */
    public boolean emptyListAllowed()
    {
        return allowEmptyList;
    }

    /**
     * Sets whether or not validation when done by this widget should allow the
     * user to remove all of the elements from the list.
     * 
     * @param allowEmptyList
     *            true if the user should be allowed to remove all of the
     *            elements from the list, false otherwise.
     * 
     */
    public void allowEmptyList(boolean allowEmptyList)
    {
        this.allowEmptyList = allowEmptyList;
    }

}