/*
 * Created on Jul 29, 2006
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import org.cafesip.gwtcomp.client.utils.HTMLHelper;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;

/**
 * A message bar is a panel that can be added to any GWT panel/form/etc and is
 * used to display informational, error, or warning messages. The message bar
 * displays only one message at a time.
 * 
 * <p>
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/MiscWidgetsSample.java.html">MiscWidgetsSample.java</a></li>
 * </ul>
 * <p>
 * <u>CSS Style Rules:</u>
 * <ul class="css">
 * <li>.gwtcomp-MessageBar { } - style for the message bar itself</li>
 * </ul>
 * 
 * <p>
 * <u>Screen-Shots:</u><br>
 * <img class='gallery' src='../../../../../../screen-shots/message_bar.gif'/>
 * 
 * @author Amit Chatterjee
 * 
 */
public class MessageBar extends Composite
{
    public static final int SEVERITY_NONE = 0;

    public static final int SEVERITY_INFORMATIONAL = 1;

    public static final int SEVERITY_WARNING = 2;

    public static final int SEVERITY_ERROR = 3;

    private FlowPanel panel;

    private HTML message;

    private int severity = SEVERITY_INFORMATIONAL;

    private String text = "[Empty]";

    private boolean audioEnabled = false;

    /**
     * A constructor for this class.
     * 
     */
    public MessageBar()
    {
        super();

        panel = new FlowPanel();
        initWidget(panel);

        init();
    }

    private void init()
    {
        setStyleName("gwtcomp-MessageBar");
        setWidth("100%");
        message = new HTML("&nbsp;");
        message.setWidth("100%");
        panel.add(message);
        panel.add(new HTML(HTMLHelper.hr("#AAAAAA")));
    }

    /**
     * Prints an informational message.
     * 
     * @param text
     *            message to print.
     */
    public void setMessage(String text)
    {
        setMessage(text, SEVERITY_INFORMATIONAL);
    }

    /**
     * Prints a message with a given severity.
     * 
     * @param text
     *            message to print.
     * @param severity
     *            of the message (see the constants defined above).
     */
    public void setMessage(String text, int severity)
    {
        this.text = text;
        this.severity = severity;

        String color = null;
        String icon = null;

        switch (severity)
        {
        case SEVERITY_NONE:
            clearMessage();
            return;

        case SEVERITY_INFORMATIONAL:
            icon = GWT.getModuleBaseURL() + "gwtcomp-icons/info.png";
            color = "blue";
            break;

        case SEVERITY_WARNING:
            icon = GWT.getModuleBaseURL() + "gwtcomp-icons/bell.png";
            color = "#FF9900";
            if (audioEnabled)
            {
                Audio.getInstance().playWarning();
            }
            break;

        case SEVERITY_ERROR:
            icon = GWT.getModuleBaseURL() + "gwtcomp-icons/no.png";
            color = "red";

            if (audioEnabled)
            {
                Audio.getInstance().playError();
            }
            break;
        }

        message.setHTML("<img border='0' align='top' src='" + icon + "'/>"
                + "&nbsp;<span style=\"color: " + color + ";\">" + text
                + "</span>");
    }

    /**
     * Clears the message
     */
    public void clearMessage()
    {
        message.setHTML("&nbsp;");
        text = null;
        severity = SEVERITY_INFORMATIONAL;
    }

    /**
     * Returns whether audio messaging is enabled.
     * 
     * @return true if audio is enabled
     */
    public boolean isAudioEnabled()
    {
        return audioEnabled;
    }

    /**
     * Sets audio messaging. If audio messaging is set, an audio is played
     * depending on the severity level. It is disabled by default.
     * 
     * @param audioEnabled
     *            true if audio is enabled.
     */
    public void setAudioEnabled(boolean audioEnabled)
    {
        this.audioEnabled = audioEnabled;
    }

    /**
     * Returns the serverity of the currently-displayed message.
     * 
     * @return Returns the severity.
     */
    public int getSeverity()
    {
        return severity;
    }

    /**
     * Sets the severity of the currently-displayed message.
     * 
     * @param severity
     *            The severity to set.
     */
    public void setSeverity(int severity)
    {
        this.severity = severity;
    }

    /**
     * Returns the text of the message being displayed.
     * 
     * @return Returns the text.
     */
    public String getText()
    {
        return text;
    }

    /**
     * Sets the text of the message.
     * 
     * @param text
     *            The text to set.
     */
    public void setText(String text)
    {
        setMessage(text, severity);
    }
}
