/*
 * Created on Aug 18, 2007
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import org.cafesip.gwtcomp.client.utils.HTMLHelper;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.MouseListenerAdapter;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * A popup menu that is used to popup and present a set of options to the users.
 * The menu can be popped up or down programatically. Applications can add
 * widgets as menu items. Therefore, many items can contain text lables, HTML
 * segments, user input fields, etc. In addition, the applications can also add
 * cascaded child menus.
 * <p>
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/SuperTreeSample.java.html"
 * >SuperTreeSample.java</a></li> </ul>
 * 
 * <p>
 * <u>CSS Style Rules:</u> <ul class="css">
 * <li>.gwtcomp-PopupMenu { } - style for the popup menu itself</li>
 * <li>.gwtcomp-PopupMenu-MenuItem { } - style for a child menu item</li>
 * <li>.gwtcomp-PopupMenu-MenuItem-MouseIn{ } - the style that is applied when
 * the user mouses over a menu item</li> </ul>
 * 
 * <p>
 * <u>Screen-Shots:</u><br>
 * <img class='gallery' src='../../../../../../screen-shots/popup_menu.gif'/>
 * <p>
 * 
 * @author Amit Chatterjee
 * 
 */
public class PopupMenu extends PopupPanel
{
    private PopupMenu parent = null;

    private VerticalPanel panel = new VerticalPanel();

    private PopupMenu openChildMenu = null;

    /**
     * A constructor for this class.
     * 
     */
    public PopupMenu()
    {
        super(true);
        setStyleName("gwtcomp-PopupMenu");

        panel = new VerticalPanel();
        panel.setVerticalAlignment(HasVerticalAlignment.ALIGN_TOP);
        panel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_LEFT);
        setWidget(panel);

    }

    /**
     * Adds a child menu item that is a cascaded menu.
     * 
     * @param widget
     *            normally a HTML or Label object is used to print the menu item
     *            name.
     * @param cascadedMenu
     *            the popup menu that is popped up when the user mouses over the
     *            widget.
     * 
     */
    public void addMenuItem(Widget widget, PopupMenu cascadedMenu)
    {
        class ItemSelectionListener extends MouseListenerAdapter
        {
            private PopupMenu parent;

            private FocusPanel panel;

            private PopupMenu cascadedMenu;

            public ItemSelectionListener(PopupMenu parent, FocusPanel panel,
                    PopupMenu cascadedMenu)
            {
                this.parent = parent;
                this.panel = panel;
                this.cascadedMenu = cascadedMenu;
            }

            public void onMouseEnter(Widget sender)
            {
                panel.addStyleName("gwtcomp-PopupMenu-MenuItem-MouseIn");

                // hide the previous cascaded menu
                if (parent.openChildMenu != null)
                {
                    parent.openChildMenu.hide();
                    parent.openChildMenu = null;
                }

                // open the cascaded menu if one is available
                if (cascadedMenu != null)
                {
                    int top = sender.getAbsoluteTop();
                    int left = sender.getAbsoluteLeft()
                            + sender.getOffsetWidth();
                    cascadedMenu.setPopupPosition(left, top);
                    cascadedMenu.show();
                    parent.openChildMenu = cascadedMenu;
                }
            }

            public void onMouseLeave(Widget sender)
            {
                panel.removeStyleName("gwtcomp-PopupMenu-MenuItem-MouseIn");
            }
        }

        HorizontalPanel w = new HorizontalPanel();
        w.addStyleName("gwtcomp-PopupMenu-MenuItem");
        w.setWidth("100%");
        w.add(widget);
        w.setCellHorizontalAlignment(widget, HasHorizontalAlignment.ALIGN_LEFT);
        w.setCellVerticalAlignment(widget, HasVerticalAlignment.ALIGN_MIDDLE);

        // add spacing
        w.add(new HTML("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"));

        if (cascadedMenu != null)
        {
            Image im = new Image(GWT.getModuleBaseURL()
                    + "gwtcomp-icons/right-arrow.png");
            w.add(im);
            w
                    .setCellHorizontalAlignment(im,
                            HasHorizontalAlignment.ALIGN_RIGHT);
            w.setCellVerticalAlignment(im, HasVerticalAlignment.ALIGN_MIDDLE);

            cascadedMenu.parent = this;
        }

        FocusPanel fp = new FocusPanel(w);
        panel.add(fp);
        fp.setWidth("100%");
        fp.addStyleName("gwtcomp-PopupMenu-MenuItem");
        fp.addMouseListener(new ItemSelectionListener(this, fp, cascadedMenu));
    }

    /**
     * Add a child menu item.
     * 
     * @param widget
     *            normally a HTML segment or a text lable is passed but other
     *            input widgets can be passed as well.
     */
    public void addMenuItem(Widget widget)
    {
        addMenuItem(widget, null);
    }

    /**
     * Removes a child menu at the given index.
     * 
     * @param index
     *            index
     */
    public void removeMenuItem(int index)
    {
        panel.remove(index);
    }

    /**
     * Add a separator item.
     */
    public void addSeparator()
    {
        panel.add(new HTML(HTMLHelper.hr("#AAAAAA")));
    }

    /**
     * Hides the popup menu. In case the menu is a cascaded menu and has
     * parents, the parents are hidden as well.
     */
    public void hideAll()
    {
        this.hide();
        PopupMenu parent = this.parent;
        while (parent != null)
        {
            parent.hide();
            parent = parent.parent;
        }

        // Don't leave any menu items selected
        int numItems = panel.getWidgetCount();
        for (int i = 0; i < numItems; i++)
        {
            panel.getWidget(i).removeStyleName(
                    "gwtcomp-PopupMenu-MenuItem-MouseIn");
        }
    }

    /**
     * Gets the root menu for a given popup menu. Cascaded menus have parents
     * and possibly grand-parents. This method iterates through the chain and
     * returns the root popup menu.
     * 
     * @param menu
     *            a given popup menu object
     * @return the root popup menu
     */
    public static PopupMenu getRootMenu(PopupMenu menu)
    {
        PopupMenu element = menu;
        PopupMenu parent = element.parent;
        while (parent != null)
        {
            element = parent;
            parent = element.parent;
        }

        return element;
    }
}
