/*
 * Created on Aug 5, 2006
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.cafesip.gwtcomp.client.utils.HTMLHelper;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Widget;

/**
 * A panel for viewing properties. A property consists of two elements - a key
 * and a value. This widget displays properties in a tabular format. Properties
 * can be sorted if desired.
 * 
 * <p>
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a
 * href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/PropertyViewerSample.java.html">PropertyViewerSample.java</a></li>
 * </ul>
 * 
 * <p>
 * <u>CSS Style Rules:</u>
 * <ul class="css">
 * <li>.gwtcomp-PropertyViewer { } - style for the property viewer itself</li>
 * <li>.gwtcomp-PropertyViewerName { } - style for the name columns of the
 * property</li>
 * <li>.gwtcomp-PropertyViewerValue { } - style for the value columns of the
 * property</li>
 * <li>.gwtcomp-PropertyViewerHeader { } - style for the header</li>
 * <li>.gwtcomp-PropertyViewerHeaderSort { } - style for the sort icons</li>
 * </ul>
 * <p>
 * 
 * <u>Screen-Shots:</u><br>
 * <img class='gallery'
 * src='../../../../../../screen-shots/property_viewer.gif'/>
 * 
 * @author Amit Chatterjee
 * 
 */
public class PropertyViewer extends Composite
{
    private FlexTable table;

    private boolean showHeader = false;

    private boolean allowSort = false;

    private ArrayList<PropertyElement> properties = new ArrayList<PropertyElement>();

    private String keyCaption = "Property";

    private String valueCaption = "Value";

    /**
     * A constructor for this class. Creates a property viewer object.
     * 
     * @param showHeader
     *            true if the header is to be displayed
     * @param allowSort
     *            true if the sort icons are to be displayed
     * @wbp.parser.constructor
     */
    public PropertyViewer(boolean showHeader, boolean allowSort)
    {
        super();
        this.showHeader = showHeader;
        this.allowSort = allowSort;

        table = new FlexTable();
        initWidget(table);

        setStyleName("gwtcomp-PropertyViewer");
        table.getFlexCellFormatter().setWidth(0, 0, "50%");

        if (showHeader == true)
        {
            setHeader();
        }
    }

    /**
     * A constructor for this class. Creates a property viewer object with
     * header and sort icons.
     * 
     */
    public PropertyViewer()
    {
        this(true, true);
    }

    /**
     * Display the header.
     */
    private void setHeader()
    {
        table.getRowFormatter().setVerticalAlign(0,
                HasVerticalAlignment.ALIGN_TOP);

        if (allowSort == false)
        {
            table.setHTML(0, 0, HTMLHelper.bold(keyCaption));
        }
        else
        {
            HorizontalPanel hp = new HorizontalPanel();
            hp.setStyleName("gwtcomp-PropertyViewerHeaderSort");
            Image asc_link = new Image(GWT.getModuleBaseURL()
                    + "gwtcomp-icons/down.png");
            asc_link.addStyleName("gwtcomp-SortArrow");
            asc_link.setTitle("Sort in ascending order");
            asc_link.addClickListener(new ClickListener()
            {
                public void onClick(Widget sender)
                {
                    sortAscending();
                }
            });

            hp.add(asc_link);

            Image dsc_link = new Image(GWT.getModuleBaseURL()
                    + "gwtcomp-icons/up.png");
            dsc_link.addStyleName("gwtcomp-SortArrow");
            dsc_link.setTitle("Sort in descending order");
            dsc_link.addClickListener(new ClickListener()
            {

                public void onClick(Widget sender)
                {
                    sortDescending();
                }
            });
            hp.add(dsc_link);
            hp.add(new HTML(HTMLHelper.bold(keyCaption)));
            table.setWidget(0, 0, hp);
        }
        table.getCellFormatter().setStyleName(0, 0,
                "gwtcomp-PropertyViewerHeader");

        table.setHTML(0, 1, HTMLHelper.bold(valueCaption));
        table.getCellFormatter().setStyleName(0, 1,
                "gwtcomp-PropertyViewerHeader");
    }

    /**
     * Add a property to be displayed.
     * 
     * @param key
     *            the property key
     * @param value
     *            a string value
     */
    public void addProperty(String key, String value)
    {
        setRow(table.getRowCount(), key, value);
        properties.add(new PropertyElement(key, value));
    }

    /**
     * Add a property to be displayed.
     * 
     * @param key
     *            the property key
     * @param widget
     *            a widget to be displayed as text (instead of a text)
     */
    public void addProperty(String key, Widget widget)
    {
        setRow(table.getRowCount(), key, widget);
        properties.add(new PropertyElement(key, widget));
    }

    /**
     * Insert a property at a given row (row starts with 0).
     * 
     * @param row
     *            row number
     * @param key
     *            the property key
     * @param value
     *            the property value
     */
    private void setRow(int row, String key, String value)
    {
        rowFormat(row, key);
        table.setHTML(row, 1, HTMLHelper.bold(value));
    }

    /**
     * Insert a property at a given row (row starts with 0).
     * 
     * @param row
     *            row number
     * @param key
     *            the property key
     * @param value
     *            the property value widget
     */
    private void setRow(int row, String key, Widget widget)
    {
        rowFormat(row, key);
        table.setWidget(row, 1, widget);
    }

    private void rowFormat(int row, String key)
    {
        table.getRowFormatter().setVerticalAlign(row,
                HasVerticalAlignment.ALIGN_TOP);
        table.getCellFormatter().setStyleName(row, 0,
                "gwtcomp-PropertyViewerName");
        table.getCellFormatter().setStyleName(row, 1,
                "gwtcomp-PropertyViewerValue");
        table.setHTML(row, 0, key);
    }

    private void clear(boolean clearList)
    {
        int count = table.getRowCount();
        for (int i = count - 1; i >= 0; i--)
        {
            table.removeRow(i);
        }

        if (clearList == true)
        {
            properties.clear();
        }

        if (showHeader == true)
        {
            setHeader();
        }
    }

    /**
     * Clear the property list.
     */
    public void clear()
    {
        clear(true);
    }

    /**
     * Sets a list of properties using a HashMap object.
     * 
     * @param p
     *            the hashmap object. The key and the values for the hashmap
     *            must be Strings.
     */
    public void setProperties(HashMap<String, String> p)
    {
        clear();

        if ((p != null) && (p.size() > 0))
        {
            Iterator<Map.Entry<String, String>> i = p.entrySet().iterator();
            while (i.hasNext() == true)
            {
                Map.Entry<String, String> entry = i.next();
                String param = entry.getKey();
                String value = entry.getValue();

                addProperty(param, value);
            }
        }
    }

    private void sortAscending()
    {
        Collections.sort(properties, new Comparator<PropertyElement>()
        {
            public int compare(PropertyElement e1, PropertyElement e2)
            {
                return e1.getProperty().compareTo(e2.getProperty());
            }
        });

        repopulate();
    }

    private void sortDescending()
    {
        Collections.sort(properties, new Comparator<PropertyElement>()
        {
            public int compare(PropertyElement e1, PropertyElement e2)
            {
                return e2.getProperty().compareTo(e1.getProperty());
            }
        });

        repopulate();
    }

    private void repopulate()
    {
        clear(false);

        int size = properties.size();
        for (int i = 0; i < size; i++)
        {
            PropertyElement e = properties.get(i);
            if (e.getValue() instanceof String)
            {
                setRow(showHeader == true ? i + 1 : i, e.getProperty(),
                        (String) e.getValue());
            }
            else
            // widget
            {
                setRow(showHeader == true ? i + 1 : i, e.getProperty(),
                        (Widget) e.getValue());
            }
        }
    }

    /**
     * Returns the caption for the Key column.
     * 
     * @return Returns the keyCaption.
     */
    public String getKeyCaption()
    {
        return keyCaption;
    }

    /**
     * Sets the caption for the key column.
     * 
     * @param keyCaption
     *            The keyCaption to set.
     */
    public void setKeyCaption(String keyCaption)
    {
        this.keyCaption = keyCaption;
        setHeader();
    }

    /**
     * Returns the caption for the value column.
     * 
     * @return Returns the valueCaption.
     */
    public String getValueCaption()
    {
        return valueCaption;
    }

    /**
     * Sets the caption for the value column.
     * 
     * @param valueCaption
     *            The valueCaption to set.
     */
    public void setValueCaption(String valueCaption)
    {
        this.valueCaption = valueCaption;
        setHeader();
    }
}
