/*
 * Created on Jul 19, 2006
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import java.util.ArrayList;
import java.util.Iterator;

import org.cafesip.gwtcomp.client.common.DirListingService;
import org.cafesip.gwtcomp.client.common.DirListingServiceAsync;
import org.cafesip.gwtcomp.client.common.DirectoryElement;
import org.cafesip.gwtcomp.client.common.FileElement;
import org.cafesip.gwtcomp.client.utils.ContextUtil;
import org.cafesip.gwtcomp.client.utils.HTMLHelper;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * Lets GWT applications browse and selects files that are present in the web
 * server hosting the GWT application. For system administration purposes, very
 * often, users have to specify a file or a directory in the server. This widget
 * lets the user browser the files in the server using a file selector user
 * interface.
 * <p>
 * Note that this component requires a GWT servlet to be running on the server.
 * You will need to include the gwtcomp-servlet.jar in the WEB-INF/web.xml file
 * and in the web.xml file, add the following segment:
 * 
 * <pre>
 *   &lt;servlet&gt;
 *        &lt;servlet-name&gt;DirListingService&lt;/servlet-name&gt;
 *        &lt;servlet-class&gt;
 *            org.cafesip.gwtcomp.server.DirListingServiceImpl
 *        &lt;/servlet-class&gt;
 *    &lt;/servlet&gt;
 *    ...
 *    &lt;servlet-mapping&gt;
 *        &lt;servlet-name&gt;DirListingService&lt;/servlet-name&gt;
 *        &lt;url-pattern&gt;/gwtcomp/directoryService&lt;/url-pattern&gt;
 *    &lt;/servlet-mapping&gt;
 * </pre>
 * 
 * <p>
 * 
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a
 * href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/ServerFileBrowserSample.java.html">ServerFileBrowserSample.java</a></li>
 * </ul>
 * 
 * <u>CSS Style Rules:</u>
 * <ul class="css">
 * <li>.gwtcomp-ServerFileBrowser { } - style for the file browser diaglog box</li>
 * <li>.gwtcomp-ServerFileBrowserList { } - style for the file list</li>
 * <li>.gwtcomp-ServerFileBrowserServerWait { } - style to be applied while the
 * file browser is waiting to hear back from the server</li> *
 * </ul>
 * 
 * <p>
 * <u>Screen-Shots:</u><br>
 * <img class='gallery'
 * src='../../../../../../screen-shots/server_file_browser.gif'/>
 * </p>
 * 
 * @author Amit Chatterjee
 * 
 */
public class ServerFileBrowser extends DialogBox
{
    private static final String DIRECTORY_SERVICE = ContextUtil.getContextURL()
            + "/gwtcomp/directoryService";

    private static final int MAX_FILE_NAME_LENGTH = 20;

    private Label dir;

    private ListBox files;

    private TextBox selection;

    private DirectoryElement currentDirectory;

    private ArrayList<ServerFileBrowserListener> listeners = new ArrayList<ServerFileBrowserListener>();

    private String filter;

    /**
     * A constructor for this class.
     * 
     * @param filter
     *            a regular expression filter String. Only files/directories
     *            matching the pattern will be displayed.
     * @wbp.parser.constructor
     * 
     */
    public ServerFileBrowser(String filter)
    {
        super();
        this.filter = filter;
        init();
    }

    /**
     * A constructor for this class. Creates a file browser with no filtering.
     * 
     */
    public ServerFileBrowser()
    {
        this(null);
    }

    /**
     * Adds a listener for events.
     * 
     * @param listener
     */
    public void addServerFileListener(ServerFileBrowserListener listener)
    {
        listeners.add(listener);
    }

    /**
     * Removes an event listener.
     * 
     * @param listener
     */
    public void removeServerFileListener(ServerFileBrowserListener listener)
    {
        listeners.remove(listener);
    }

    private void init()
    {
        VerticalPanel panel = new VerticalPanel();
        panel.setStyleName("gwtcomp-ServerFileBrowser");

        panel.add(new HTML(HTMLHelper.bold("Folder:")));
        dir = new Label("Root");
        panel.add(dir);

        panel.add(new HTML(HTMLHelper.newline() + HTMLHelper.bold("Files:")));

        ScrollPanel sp = new ScrollPanel();
        sp.setWidth("400px");
        sp.setHeight("200px");
        files = new ListBox();
        files.setWidth("800px");
        // files.setHeight("100%");

        files.setStyleName("gwtcomp-ServerFileBrowserList");
        populateFiles(null);
        files.setVisibleItemCount(10);

        files.addChangeListener(new ChangeListener()
        {
            public void onChange(Widget sender)
            {
                int index = files.getSelectedIndex();
                if (index >= 0)
                {
                    String s = currentDirectory.getDirectory()
                            + currentDirectory.getSeparator()
                            + currentDirectory.getFiles()[index].getName();
                    if (currentDirectory.getFiles()[index].getType().equals(
                            FileElement.DIR) == true)
                    {
                        populateFiles(s);
                        return;
                    }

                    selection.setText(s);
                    // selection.setVisibleLength(s.length());
                }
            }
        });

        sp.add(files);
        panel.add(sp);

        panel
                .add(new HTML(HTMLHelper.newline()
                        + HTMLHelper.bold("Selection:")));
        selection = new TextBox();
        selection.setWidth("100%");
        panel.add(selection);

        panel.add(new HTML(HTMLHelper.hr("#999999")));

        HorizontalPanel hp = new HorizontalPanel();
        hp.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
        hp.setSpacing(10);
        Button up = new Button(HTMLHelper.imageWithText(GWT.getModuleBaseURL()
                + "gwtcomp-icons/up.png", " Up "));
        up.addClickListener(new ClickListener()
        {

            public void onClick(Widget sender)
            {
                if (currentDirectory == null)
                {
                    return;
                }

                String s = currentDirectory.getDirectory() + "/..";
                populateFiles(s);
            }
        });

        hp.add(up);
        Button ok = new Button(HTMLHelper.imageWithText(GWT.getModuleBaseURL()
                + "gwtcomp-icons/ok.png", " Ok "));
        ok.addClickListener(new ClickListener()
        {

            public void onClick(Widget sender)
            {
                Iterator<ServerFileBrowserListener> i = listeners.iterator();
                while (i.hasNext() == true)
                {
                    ServerFileBrowserListener listener = i.next();
                    listener.onOk(ServerFileBrowser.this, selection.getText()
                            .trim());
                }
            }
        });

        hp.add(ok);
        Button cancel = new Button(HTMLHelper.imageWithText(GWT
                .getModuleBaseURL()
                + "gwtcomp-icons/cancel.png", " Cancel "));
        cancel.addClickListener(new ClickListener()
        {
            public void onClick(Widget sender)
            {
                Iterator<ServerFileBrowserListener> i = listeners.iterator();
                while (i.hasNext() == true)
                {
                    ServerFileBrowserListener listener = i.next();
                    listener.onCancel(ServerFileBrowser.this);
                }
            }
        });
        hp.add(cancel);
        panel.add(hp);

        setText("Server File Browser");
        setWidget(panel);
    }

    private void populateFiles(String directory)
    {
        files.addStyleName("gwtcomp-ServerFileBrowserServerWait");
        addStyleName("gwtcomp-ServerFileBrowserServerWait");
        DirListingServiceAsync ctx = (DirListingServiceAsync) GWT
                .create(DirListingService.class);
        ServiceDefTarget endpoint = (ServiceDefTarget) ctx;
        endpoint.setServiceEntryPoint(DIRECTORY_SERVICE);

        ctx.list(directory, filter, new AsyncCallback<DirectoryElement>()
        {
            public void onSuccess(DirectoryElement res)
            {
                try
                {
                    currentDirectory = res;
                    FileElement[] elements = currentDirectory.getFiles();
                    files.clear();

                    for (int i = 0; i < elements.length; i++)
                    {
                        files.addItem(format(elements[i]));
                    }

                    if (elements.length < 10)
                    {
                        files.setVisibleItemCount(10);
                    }
                    else
                    {
                        files.setVisibleItemCount(elements.length);
                    }

                    if (currentDirectory.getDirectory() != null)
                    {
                        dir.setText(currentDirectory.getDirectory());
                        selection.setText(currentDirectory.getDirectory());
                    }
                }
                finally
                {
                    files
                            .removeStyleName("gwtcomp-ServerFileBrowserServerWait");
                    removeStyleName("gwtcomp-ServerFileBrowserServerWait");
                }
            }

            public void onFailure(Throwable caught)
            {
                removeStyleName("gwtcomp-ServerFileBrowserServerWait");
                Window.alert("An error occured: " + caught.getMessage());
            }
        });
    }

    private String format(FileElement element)
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append("[" + element.getPermissions() + "] ");

        String name = element.getName();
        if (name.length() > MAX_FILE_NAME_LENGTH)
        {
            buffer.append(name.substring(0, MAX_FILE_NAME_LENGTH - 3) + "...");
        }
        else
        {
            buffer.append(name);
            for (int i = name.length(); i < MAX_FILE_NAME_LENGTH; i++)
            {
                buffer.append(" ");
            }
        }

        buffer.append("  (" + element.getLastModified() + ")");
        return buffer.toString();
    }

    /**
     * Returns the filter string.
     * 
     * @return Returns the filter.
     */
    public String getFilter()
    {
        return filter;
    }

    /**
     * Sets a filter string.
     * 
     * @param filter
     *            The filter to set.
     */
    public void setFilter(String filter)
    {
        this.filter = filter;
    }
}
