/*
 * Created on Aug 18, 2007
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.Tree;
import com.google.gwt.user.client.ui.TreeImages;
import com.google.gwt.user.client.ui.TreeItem;
import com.google.gwt.user.client.ui.TreeListener;
import com.google.gwt.user.client.ui.Widget;

/**
 * An extension of the GWT Tree that allows multiple selection of tree items and
 * includes comprehensive support for context-sensitive popup menu among other
 * things. The main feature of the objects is the ability for the application to
 * define different popup menus at every unique level of the tree. For example,
 * if a tree is displaying countries, its states and cities within a state, it
 * is possible to define a different popup menu at the country-level,
 * state-level and city-level without having to attach a popup menu for
 * individual items although that is possible as well. In order to utilize the
 * feature, the application must use
 * {@link org.cafesip.gwtcomp.client.ui.SuperTreeItem} instead of the GWT
 * TreeItem object to define the tree hierarchy. Each object of the type
 * SuperTreeItem has an integer "type" attribute that must be specified. A level
 * is specified as a string formatted as W.X.Y.Z.... where W, X, Y, Z are types.
 * The dots are used to specify the hierarchy. For example, if the country has
 * the type 1, state has the type 2 and the city has the type 3, the level 1.2.3
 * is used to specify a context menu for the city that is placed in the tree as
 * Country->State->City. The context-menu is popped up by using the right-click.
 * <b>Note that the application must not use the type value of 0. That value is
 * reserved for objects of the type TreeItem. </b>
 * <p>
 * If the tree has multi-selection enabled, the user can select more than one
 * tree item by clicking on the tree item while pressing the CTRL key. The user
 * can select multiple items belonging to the same level only. Therefore, in
 * conjuction with the the context-senstive popup menu, the user can select
 * multiple items of the same level and popup the menu.
 * <p>
 * The application can also define their own tree listener for listening to tree
 * selection, tree expand and collapse and popup events. For more details, see
 * the {@link org.cafesip.gwtcomp.client.ui.SuperTreeListener} class.
 * <p>
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a
 * href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/SuperTreeSample.java.html">SuperTreeSample.java</a></li>
 * </ul>
 * 
 * <p>
 * <u>CSS Style Rules:</u>
 * <ul class="css">
 * <li>.gwtcomp-SuperTree { } - style for the tree itself</li>
 * <li>.gwtcomp-SuperTree-SelectedItem { } - style for a selected tree item</li>
 * </ul>
 * 
 * <p>
 * <u>Screen-Shots:</u><br>
 * <img class='gallery' src='../../../../../../screen-shots/super_tree.gif'/>
 * <p>
 * 
 * @author Amit Chatterjee
 * 
 */
public class SuperTree extends Tree
{
    private HashMap<String, PopupMenu> contextMenus = new HashMap<String, PopupMenu>();

    private boolean ctrlKeyPressed = false;

    private ArrayList<SuperTreeListener> listeners = new ArrayList<SuperTreeListener>();

    private boolean multiSelectionEnabled = false;

    private boolean rightClicked = false;

    private ArrayList<TreeItem> selectedItems = new ArrayList<TreeItem>();

    private String selectedLevel = "";

    /**
     * A constructor for this class.
     * 
     */
    public SuperTree()
    {
        super();
        init();
    }

    /**
     * A constructor for this class.
     * 
     * @param images
     *            the image bundle.
     */
    public SuperTree(TreeImages images)
    {
        super(images);
        init();
    }

    protected static native void preventBrowserContextMenu(
            JavaScriptObject element)
    /*-{
     element.oncontextmenu = function() 
     {
     return false;
     }
     }-*/;

    /**
     * @see com.google.gwt.user.client.ui.Tree#addItem(java.lang.String).
     *      <p>
     *      Note that all the items in the tree are converted to GWT HTML object
     *      before adding to the tree. Therefore, the GWT Tree.getText() method
     *      will return a null.
     */
    public TreeItem addItem(String itemText)
    {
        HTML html = new HTML(itemText);
        preventBrowserContextMenu(html.getElement());
        return super.addItem(html);
    }

    /**
     * Add a SuperTree object to the root of the tree.
     * 
     * @param item
     *            item to add.
     */
    public void addItem(SuperTreeItem item)
    {
        super.addItem(item);
        preventBrowserContextMenu(item.getElement());
    }

    /**
     * @see com.google.gwt.user.client.ui.Tree#addItem(com.google.gwt.user.client.ui.TreeItem)
     */
    public void addItem(TreeItem item)
    {
        super.addItem(item);
        preventBrowserContextMenu(item.getElement());
    }

    /**
     * @see com.google.gwt.user.client.ui.Tree#addItem(com.google.gwt.user.client.ui.Widget)
     */
    public TreeItem addItem(Widget widget)
    {
        preventBrowserContextMenu(widget.getElement());
        return super.addItem(widget);
    }

    /**
     * Add a tree listener.
     * 
     * @param listener
     *            the listener object.
     */
    public void addTreeListener(SuperTreeListener listener)
    {
        listeners.add(listener);
    }

    /**
     * Convenience method for finding the level of any given tree items. Note
     * that although an object of the type TreeItem is passed as a parameter,
     * normally, an object of the type SuperTreeItem must be used. An object of
     * the type TreeItem is assumed to have the tye 0.
     * 
     * @param item
     *            a tree item.
     * @return the level
     */
    public static String getLevel(TreeItem item)
    {
        ArrayList<Integer> items = new ArrayList<Integer>();
        do
        {
            if (item instanceof SuperTreeItem)
            {
                items.add(new Integer(((SuperTreeItem) item).getType()));
            }
            else
            {
                items.add(new Integer(0));
            }

            item = item.getParentItem();
        }
        while (item != null);

        StringBuffer buffer = new StringBuffer();
        Collections.reverse(items);

        for (int i = 0; i < items.size(); i++)
        {
            if (i > 0)
            {
                buffer.append('.');
            }

            buffer.append(((Integer) items.get(i)).toString());
        }
        return buffer.toString();
    }

    /**
     * @see com.google.gwt.user.client.ui.Tree#getSelectedItem().
     * 
     * <p>
     * Note that if multi-selection is enabled, this method returns the item
     * selected first. For trees with multi-selection, the getSelectedItems()
     * method must be used.
     * 
     */
    public TreeItem getSelectedItem()
    {
        if (selectedItems.size() == 0)
        {
            return null;
        }

        return selectedItems.get(0);
    }

    /**
     * Returns the selected items.
     * 
     * @return selected items.
     */
    public TreeItem[] getSelectedItems()
    {
        TreeItem[] items = convertSelectedToArray();
        return items;
    }

    /**
     * Returns if multi-selection is enabled.
     * 
     * @return Returns the multiSelectionEnabled.
     */
    public boolean isMultiSelectionEnabled()
    {
        return multiSelectionEnabled;
    }

    /*
     * @see com.google.gwt.user.client.ui.Tree#onBrowserEvent(com.google.gwt.user.client.Event)
     */
    public void onBrowserEvent(Event event)
    {
        switch (DOM.eventGetType(event))
        {
        case Event.ONKEYDOWN:
            int key = DOM.eventGetKeyCode(event);
            if (key == KeyboardListener.KEY_CTRL)
            {
                ctrlKeyPressed = true;
            }
            break;

        case Event.ONKEYUP:
            key = DOM.eventGetKeyCode(event);
            if (key == KeyboardListener.KEY_CTRL)
            {
                ctrlKeyPressed = false;
            }
            break;

        case Event.ONMOUSEDOWN:
            if (DOM.eventGetButton(event) == Event.BUTTON_RIGHT)
            {
                rightClicked = true;
            }
            break;

        case Event.ONMOUSEUP:
            if (DOM.eventGetButton(event) == Event.BUTTON_RIGHT)
            {
                rightClicked = false;
            }
            break;
        }

        super.onBrowserEvent(event);
    }

    /**
     * Remove the context menu at a given level.
     * 
     * @param level
     */
    public void removeContextMenu(String level)
    {
        contextMenus.remove(level);
    }

    /**
     * Removes the tree listener.
     * 
     * @param listener
     *            listener to remove.
     */
    public void removeTreeListener(SuperTreeListener listener)
    {
        listeners.remove(listener);
    }

    /**
     * Sets the context menu for a given level.
     * 
     * @param level
     *            the level to set
     * @param contextMenu
     *            the context menu object.
     */
    public void setContextMenu(String level, PopupMenu contextMenu)
    {
        contextMenus.put(level, contextMenu);
    }

    /**
     * Enables or disables multi-selection.
     * 
     * @param multiSelectionEnabled
     *            The multiSelectionEnabled to set.
     */
    public void setMultiSelectionEnabled(boolean multiSelectionEnabled)
    {
        this.multiSelectionEnabled = multiSelectionEnabled;
    }

    /**
     * 
     * @see com.google.gwt.user.client.ui.Tree#setSelectedItem(com.google.gwt.user.client.ui.TreeItem)
     */
    public void setSelectedItem(TreeItem item)
    {
        setSelectedItem(item, false);
    }

    /**
     * @see com.google.gwt.user.client.ui.Tree#setSelectedItem(com.google.gwt.user.client.ui.TreeItem,
     *      boolean)
     */
    public void setSelectedItem(TreeItem item, boolean fireEvents)
    {
        if (item == null)
        {
            // let the GWT tree handle deselection of an item being removed
            super.setSelectedItem(item, fireEvents);
            return;
        }

        if (fireEvents)
        {
            // let the GWT tree do its work. This is needed because the
            // application
            // may have registered one or more TreeListener.
            super.setSelectedItem(item, fireEvents);
        }
        else
        {
            processItemSelected(item, false);
        }
    }

    private TreeItem[] convertSelectedToArray()
    {
        TreeItem[] items = new TreeItem[selectedItems.size()];
        for (int i = 0; i < items.length; i++)
        {
            items[i] = selectedItems.get(i);
        }
        return items;
    }

    private void init()
    {
        addStyleName("gwtcomp-SuperTree");
        initTreeListener();
    }

    private void initTreeListener()
    {
        addTreeListener(new TreeListener()
        {
            public void onTreeItemSelected(TreeItem item)
            {
                processItemSelected(item, true);
            }

            public void onTreeItemStateChanged(TreeItem item)
            {
                for (int i = 0; i < listeners.size(); i++)
                {
                    SuperTreeListener listener = listeners.get(i);

                    if (item.getState())
                    {
                        listener.onExpand(item);
                    }
                    else
                    {
                        listener.onCollapse(item);
                    }
                }
            }
        });
    }

    private void popup(TreeItem parent, PopupMenu menu, String level)
    {
        if (rightClicked == false)
        {
            return;
        }

        if (listeners.size() > 0)
        {
            TreeItem[] items = convertSelectedToArray();
            for (int i = 0; i < listeners.size(); i++)
            {
                SuperTreeListener listener = listeners.get(i);
                listener.onPopup(level, menu, items);
            }
        }

        int left = parent.getAbsoluteLeft() + 40;
        int top = parent.getAbsoluteTop() + 20;
        menu.setPopupPosition(left, top);
        menu.show();
    }

    private void processItemSelected(TreeItem item, boolean fireEvents)
    {
        String level = getLevel(item);
        if (processSelection(item, level) == false)
        {
            return; // an items has been unselected, no need to take any further
            // action
        }

        if (selectedItems.size() == 0)
        {
            return;
        }

        if (!selectedItems.contains(item))
        {
            return;
        }

        // check for any item-specific popup
        if (item instanceof SuperTreeItem)
        {
            SuperTreeItem sitem = (SuperTreeItem) item;
            if (sitem.getContextMenu() != null)
            {
                popup(item, sitem.getContextMenu(), level);
                return;
            }
        }

        // check for any level-specific popup
        PopupMenu menu = contextMenus.get(level);
        if (menu != null)
        {
            popup(item, menu, level);
            return;
        }

        // send the browser event
        if (listeners.size() == 0)
        {
            return;
        }

        if (fireEvents)
        {
            TreeItem[] items = convertSelectedToArray();
            for (int i = 0; i < listeners.size(); i++)
            {
                SuperTreeListener listener = listeners.get(i);
                listener.onSelect(level, items);
            }
        }
    }

    private boolean processSelection(TreeItem item, String level)
    {
        if (multiSelectionEnabled)
        {
            if ((selectedItems.size() > 0) && (rightClicked)
                    && level.equals(selectedLevel))
            {
                return true;
            }

            if (ctrlKeyPressed)
            {
                if (selectedItems.contains(item))
                {
                    // de-select the item
                    selectedItems.remove(item);
                    item.getWidget().removeStyleName(
                            "gwtcomp-SuperTree-SelectedItem");
                    return false;
                }

                if (!level.equals(selectedLevel))
                {
                    unselectItems();
                }

                selectItem(item, level);
                return true;
            }
            else
            {
                unselectItems();
                selectItem(item, level);
                return true;
            }
        }
        else
        {
            unselectItems();
            selectItem(item, level);
            return true;
        }
    }

    private void selectItem(TreeItem item, String level)
    {
        selectedItems.add(item);
        selectedLevel = level;
        item.getWidget().addStyleName("gwtcomp-SuperTree-SelectedItem");
    }

    private void unselectItems()
    {
        for (int i = 0; i < selectedItems.size(); i++)
        {
            TreeItem el = selectedItems.get(i);
            el.getWidget().removeStyleName("gwtcomp-SuperTree-SelectedItem");
        }
        selectedItems.clear();
        selectedLevel = "";
    }
}
