/*
 * Created on Sep 5, 2007
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.TreeItem;
import com.google.gwt.user.client.ui.Widget;

/**
 * This class extends the GWT TreeItem class and is used for specifying the object type
 * required for the {@link org.cafesip.gwtcomp.client.ui.SuperTree}'s context-menu feature.
 * In addition, it also allows applications to specify a popup menu specific to this tree item.
 * If the application requires to override the level-specific context menu for this tree item,
 * it can use the setContextMenu() method to specify a different context menu.
 * 
 * @author Amit Chatterjee
 * 
 */
public class SuperTreeItem extends TreeItem
{
    private PopupMenu contextMenu;

    private int type;

    private Object userObject;

    /**
     *  A constructor for this class.
     * 
     *  @param type type of the object. <b>The object type must be greater than zero as zero is
     *  a reserved type.</b>
     */
    public SuperTreeItem(int type)
    {
        super();
        this.type = type;
    }

    /**
     * A constructor for this class.
     * 
     * @param html tree item rendition using HTML.
     * @param type type of the object. <b>The object type must be greater than zero as zero is
     *  a reserved type.</b>
     */
    public SuperTreeItem(String html, int type)
    {
        super(new HTML(html));
        this.type = type;
    }

    /**
     * A constructor for this class.
     * 
     * @param widget widget rendition of the tree item.
     * @param type type of the object. <b>The object type must be greater than zero as zero is
     *  a reserved type.</b>
     */
    public SuperTreeItem(Widget widget, int type)
    {
        super(widget);
        this.type = type;
    }

    /**
     * @see com.google.gwt.user.client.ui.TreeItem#addItem(java.lang.String).
     * <p>
     * Note that all the items in the tree are converted to GWT HTML object
     *      before adding to the tree. Therefore, the GWT Tree.getText() method
     *      will return a null.
     */
    public TreeItem addItem(String itemText)
    {
        HTML html = new HTML(itemText);
        SuperTree.preventBrowserContextMenu(html.getElement());
        return super.addItem(html);
    }

    /**
     * Add a SuperTree object as a child of this tree element.
     * 
     * @param item
     *            item to add.
     */
    public void addItem(SuperTreeItem item)
    {
        super.addItem(item);
        SuperTree.preventBrowserContextMenu(item.getElement());
    }

    /**
     * @see com.google.gwt.user.client.ui.TreeItem#addItem(com.google.gwt.user.client.ui.TreeItem)
     */
    public void addItem(TreeItem item)
    {
        SuperTree.preventBrowserContextMenu(item.getElement());
        super.addItem(item);
    }

    /**
     * @see com.google.gwt.user.client.ui.TreeItem#addItem(com.google.gwt.user.client.ui.Widget)
     */
    public TreeItem addItem(Widget widget)
    {
        SuperTree.preventBrowserContextMenu(widget.getElement());
        return super.addItem(widget);
    }

    /**
     * Gets the context menu for this element. If an element-specific context menu is not
     * set, a null is returned.
     * 
     * @return Returns the contextMenu.
     */
    public PopupMenu getContextMenu()
    {
        return contextMenu;
    }

    /**
     * Returns the type of the object.
     * 
     * @return Returns the type.
     */
    public int getType()
    {
        return type;
    }

    /**
     * @see com.google.gwt.user.client.ui.TreeItem#getUserObject()
     */
    public Object getUserObject()
    {
        return userObject;
    }

    /**
     * Sets an item-specific context menu. This will override the level-specific
     * context menu.
     * 
     * @param contextMenu
     *            The contextMenu to set.
     */
    public void setContextMenu(PopupMenu contextMenu)
    {
        this.contextMenu = contextMenu;
    }

    /**
     * @see com.google.gwt.user.client.ui.TreeItem#setHTML(java.lang.String)
     * <p>
     * Note that all the items in the tree are converted to GWT HTML object
     *      before adding to the tree. Therefore, the GWT Tree.getHTML() method
     *      will return a null.
     */
    public void setHTML(String html)
    {
        HTML h = new HTML(html);
        super.setWidget(h);
        SuperTree.preventBrowserContextMenu(h.getElement());
    }

    /**
     * @see com.google.gwt.user.client.ui.TreeItem#setText(java.lang.String)
     * <p>
     * Note that all the items in the tree are converted to GWT HTML object
     *      before adding to the tree. Therefore, the GWT Tree.getText() method
     *      will return a null.
     */
    public void setText(String text)
    {
        HTML html = new HTML(text);
        super.setWidget(html);
        SuperTree.preventBrowserContextMenu(html.getElement());
    }

    /**
     * Sets the type of this object.
     * 
     * @param type
     *            The type to set.
     */
    public void setType(int type)
    {
        this.type = type;
    }

    /**
     * @see com.google.gwt.user.client.ui.TreeItem#setUserObject(java.lang.Object)
     */
    public void setUserObject(Object userObject)
    {
        this.userObject = userObject;
    }

    /**
     * @see com.google.gwt.user.client.ui.TreeItem#setWidget(com.google.gwt.user.client.ui.Widget)
     */
    public void setWidget(Widget newWidget)
    {
        SuperTree.preventBrowserContextMenu(newWidget.getElement());
        super.setWidget(newWidget);
    }
}
