/*
 * Created on Aug 18, 2007
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *  http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui;

import java.util.ArrayList;

import org.cafesip.gwtcomp.client.helpers.ToolbarSplitterInfo;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Widget;

/**
 * A toolbar panel is container for one or more
 * {@link org.cafesip.gwtcomp.client.ui.Toolbar} objects. Many applications
 * require a toolbar panel (normally below the menu bar) where multiple toolbars
 * are placed. There is a vertical splitter separating the toolbars which can be
 * dragged to resize the toolbars inside the panel. Note that the toolbar panel
 * can be placed anywhere in the screen, not just the top.
 * 
 * <p>
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a
 * href="../../../../../../examples/org/cafesip/gwtcomp/examples/client/MiscWidgetsSample.java.html">MiscWidgetsSample.java</a></li>
 * </ul>
 * 
 * <p>
 * <u>CSS Style Rules:</u>
 * <ul class="css">
 * <li>.gwtcomp-ToolbarPanel { } - style for the panel itself</li>
 * <li>.gwtcomp-ToolbarPanel-Splitter { } - style for any splitter</li>
 * </ul>
 * 
 * <p>
 * <u>Screen-Shots:</u><br>
 * <img class='gallery' src='../../../../../../screen-shots/toolbar_panel.gif'/>
 * <p>
 * 
 * @author Amit Chatterjee
 * 
 */
public class ToolbarPanel extends AbsolutePanel
{
    private ArrayList<Toolbar> elements = new ArrayList<Toolbar>();

    private ArrayList<ToolbarSplitterInfo> splitters = new ArrayList<ToolbarSplitterInfo>();

    private boolean dragged = false;

    private ToolbarSplitterInfo draggedToolbar;

    /**
     * A constructor for this class.
     * 
     */
    public ToolbarPanel()
    {
        setStyleName("gwtcomp-ToolbarPanel");
        setWidth("100%");
        sinkEvents(Event.MOUSEEVENTS);
    }

    /**
     * Add a toolbar to the panel. The toolbars are displayed in the order they
     * are added.
     * 
     * @param bar
     *            toolbar widget
     */
    public void addToolbar(Toolbar bar)
    {
        if (elements.size() > 0)
        {
            Toolbar tb = elements.get(elements.size() - 1);
            createSplitter(tb);
        }

        add(bar);
        elements.add(bar);
        bar.redraw();
        populate();
    }

    private Image createSplitter(Toolbar toolbar)
    {
        Image splitter = createSplitter();
        add(splitter);
        splitters.add(new ToolbarSplitterInfo(toolbar, splitter));
        return splitter;
    }

    private Image createSplitter()
    {
        Image splitter = new Image(GWT.getModuleBaseURL() + "gwtcomp-icons/spacer.png");
        splitter.setSize("4px", "16px");
        splitter.setStyleName("gwtcomp-ToolbarPanel-Splitter");
        return splitter;
    }

    /*
     * @see com.google.gwt.user.client.ui.Widget#onBrowserEvent(com.google.gwt.user.client.Event)
     */
    public void onBrowserEvent(Event event)
    {
        switch (DOM.eventGetType(event))
        {
        case Event.ONMOUSEDOWN:
        {
            Element target = DOM.eventGetTarget(event);
            int size = splitters.size();
            for (int i = 0; i < size; i++)
            {
                ToolbarSplitterInfo splitter = splitters
                        .get(i);
                Image sp = splitter.getSplitter();

                if (DOM.isOrHasChild(sp.getElement(), target))
                {
                    draggedToolbar = splitter;
                    dragged = true;
                    DOM.setCapture(getElement());
                    DOM.eventPreventDefault(event);

                    return;
                }
            }

            dragged = false;
            break;
        }

        case Event.ONMOUSEUP:
        {
            DOM.releaseCapture(getElement());
            dragged = false;
            break;
        }

        case Event.ONMOUSEMOVE:
        {
            if (dragged)
            {
                assert DOM.getCaptureElement() != null;

                Toolbar prev = draggedToolbar.getToobar();

                int x = DOM.eventGetClientX(event);

                int pw = prev.getOffsetWidth();
                int pend = prev.getAbsoluteLeft() + pw;
                int pchange = x - pend;
                int pnw = pw + pchange;
                if (pnw < 0)
                {
                    pnw = 0;
                }

                int size = elements.size();
                int[] widths = new int[size];
                for (int i = 0; i < size; i++)
                {
                    Toolbar tb = elements.get(i);
                    if (tb == prev)
                    {
                        widths[i] = pnw;
                    }
                    else
                    {
                        widths[i] = tb.getOffsetWidth();
                    }
                }

                repopulate(widths);

                DOM.eventPreventDefault(event);
            }

            break;
        }
        }
    }

    private void repopulate(int[] widths)
    {
        int size = elements.size();
        for (int i = 0; i < size; i++)
        {
            Toolbar tb = elements.get(i);
            tb.setWidth(widths[i] + "px");
        }

        populate();
    }

    /**
     * Redraw the panel. Sometimes when the size of the child toolbar changes,
     * it is necessary to draw the panel.
     */
    public void redraw()
    {
        int size = elements.size();
        for (int i = 0; i < size; i++)
        {
            Toolbar bar = elements.get(i);
            bar.redraw();
        }
        populate();
    }

    private void populate()
    {
        int max = 0;
        int left = 2;
        int top = 5;
        int size = elements.size();
        for (int i = 0; i < size; i++)
        {
            Widget w = elements.get(i);
            if (w.getOffsetHeight() > max)
            {
                max = w.getOffsetHeight();
            }
        }

        for (int i = 0; i < size; i++)
        {
            Widget w = elements.get(i);
            setWidgetPosition(w, left, top);
            w.setHeight(max + "px");
            left += w.getOffsetWidth();

            if (i < size - 1)
            {
                Image split = splitters.get(i)
                        .getSplitter();
                split.setHeight((max + top + top) + "px");
                setWidgetPosition(split, left, 0);
                left += split.getOffsetWidth();
            }

            left += 2;
        }

        setHeight((max + top + top) + "px");
        // System.out.println("toolbar panel h=" + max);
    }
    
    protected void onLoad()
    {
        super.onLoad();        
        redraw();
    }
}
