/*
 * Created on Mar 23, 2008
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.ui.charting;

import java.util.Date;

import org.cafesip.gwtcomp.client.common.ChartData;
import org.cafesip.gwtcomp.client.common.ChartInitData;
import org.cafesip.gwtcomp.client.common.LiveDataService;
import org.cafesip.gwtcomp.client.common.TimeSeriesPlotData;
import org.cafesip.gwtcomp.client.common.TimeSeriesPlotInfo;
import org.cafesip.gwtcomp.client.ui.ApplicationBusy;
import org.cafesip.gwtcomp.client.ui.ColumnProperty;
import org.cafesip.gwtcomp.client.ui.MessageBar;
import org.cafesip.gwtcomp.client.ui.SuperTable;
import org.cafesip.gwtcomp.client.ui.SuperTableProperty;

import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;

/**
 * Enables GWT application to display live data in the form of charts and
 * tables. The chart display dynamic data that is updated from the server side
 * at a pre-defined intervals. Such panels are useful for displaying changing
 * information such as stock values, temperature, etc. In order to display live
 * charts, GWT applications have to instantiate an object of this type and add
 * it to a panel of the GWT application you are developing. In addition, you
 * need to provide a server-side class(es) that contain the logic to provide the
 * chart configurations and the data needed to populate the charts periodically.
 * Each class handles a single chart that is identified by a context (see
 * below). The GWTCOMP server-side framework will instantiate these classes and
 * invoke methods on them periodically to fetch the data. The fetched data is
 * then transferred to the client-side which displays the data on the panels.
 * Note that the communication between the client and the server components is
 * transparent and your application will not have to deal with it. The data
 * source classes must implement the
 * {@link org.cafesip.gwtcomp.server.LiveDataSource} interface or extend a
 * convenience class like
 * {@link org.cafesip.gwtcomp.server.SimpleLiveDataSource} (more such classes
 * will be added in future releases). A GWT application may contain multiple
 * live data panels. Each data panel is identified by there parameters:
 * <ol>
 * <li>context - may be used to identify the type of chart, For example -
 * "STOCKS". You application may have multiple contexts like "STOCKS", "BONDS",
 * "FUTURES", etc. As explained above, you can provide a data source for every
 * context.</li>
 * <li>instance - may be used to identify a specific instance. For example -
 * "STOCKS-US".</li>
 * <li>param - an additional parameter to specify additional parameters. For
 * example - "GOOG, MSFT"</li>
 * </ol>
 * When the data source is invoked by the GWTCOMP framework, these parameters
 * are passed to the method being invoked. You application can use these
 * parameters to locate the data from a database or other data sources.
 * <p>
 * In order for this to work, you will need to add the following elements to the
 * web.xml file for your application:
 * 
 * <pre>
 *  &lt;servlet&gt;
 *  &lt;servlet-name&gt;LiveDataService&lt;/servlet-name&gt;
 *  &lt;servlet-class&gt;
 *  org.cafesip.gwtcomp.server.LiveDataServiceImpl
 *  &lt;/servlet-class&gt;
 *  &lt;init-param&gt;
 *  &lt;param-name&gt;data-source-class-contextName&lt;/param-name&gt;
 *  &lt;param-value&gt;org.cafesip.gwtcomp.examples.server.LiveDataSourceSample&lt;/param-value&gt;
 *  &lt;/init-param&gt;
 *  &lt;/servlet&gt;
 * 
 *  &lt;servlet-mapping&gt;
 *  &lt;servlet-name&gt;LiveDataService&lt;/servlet-name&gt;
 *  &lt;url-pattern&gt;/gwtcomp/liveDataService&lt;/url-pattern&gt;
 *  &lt;/servlet-mapping&gt;
 * 
 * </pre>
 * 
 * Note the use of the parameter data-source-class-contextName. Replace the
 * contextName with the name of the context. This is the context that you will
 * specify in the constructor parameters. In addition, replace the value of the
 * init-param, data-source-class with the fully qualified class name of your
 * data source class. This class must be present in the server classpath. You
 * can have multiple data-source-class-contextName init parameters, one for each
 * context. In addition, you can add an init parameter data-source-class (no
 * context) to specify the default data source class. When the GWTCOMP server
 * side receives a request with an context that is not specified in the web.xml,
 * it lets the default data source class handle the request.
 * 
 * <p>
 * The following jars must be present in your classpath in order to compile the
 * client-side of the GWT application. These jars must be in the classpath
 * regardless of whether you are using LiveDataPanel or not because of the way
 * the GWT compiler works. <ul class="css">
 * <li>gchart.jar</li> </ul>
 * 
 * <p>
 * <u>Examples:</u><br>
 * <ul class="example">
 * <li><a href="../../../../../../../examples/org/cafesip/gwtcomp/examples/client/LiveDataPanelSample.java.html"
 * > LiveDataPanelSample.java (client)</a></li>
 * <li><a href="../../../../../../../examples/org/cafesip/gwtcomp/examples/server/LiveDataSourceSample.java.html"
 * > LiveDataSourceSample.java (server)</a></li> </ul>
 * <p>
 * 
 * <p>
 * <u>CSS Style Rules:</u> <ul class="css">
 * <li>.gwtcomp-LiveDataPanel { } - style for the LiveDataPanel</li> </ul>
 * <p>
 * 
 * <p>
 * <u>Screen-Shots:</u><br>
 * <img class='gallery'
 * src='../../../../../../../screen-shots/live_data_panel.gif'/>
 * </p>
 * 
 * @author Amit Chatterjee
 * 
 */
public class LiveDataPanel extends FlowPanel
{
    public static final double GOLDEN_RATIO = 2.618;

    private static final int CHART_WIDTH = 450;

    private static final int CHART_HEIGHT = (int) (CHART_WIDTH / GOLDEN_RATIO);

    private String context;

    private String param;

    private int width = CHART_WIDTH;

    private int height = CHART_HEIGHT;

    private TimeSeriesChartPanel chartPanel;

    private String instance;

    private Timer timer;

    private long refresh;

    private boolean timerStarted = false;

    private SuperTable table;

    private int sessionId = -1;

    /**
     * 
     * A constructor for this class.
     * 
     * @param context
     *            see above
     * @param instance
     *            see above
     * @param param
     *            see above
     */
    public LiveDataPanel(String context, String instance, String param)
    {
        this(context, instance, param, CHART_WIDTH, CHART_HEIGHT);
    }

    /**
     * 
     * A constructor for this class.
     * 
     * @param context
     *            see above
     * @param instance
     *            see above
     * @param param
     *            see above
     * @param width
     *            width of the panel
     * @param height
     *            height of the panel
     */
    public LiveDataPanel(String context, String instance, String param,
            int width, int height)
    {
        super();
        setStyleName("gwtcomp-LiveDataPanel");
        setWidth("100%");

        this.width = width;
        this.height = height;
        this.context = context;
        this.instance = instance;
        this.param = param;

        init();
    }

    /**
     * 
     * A constructor for this class.
     * 
     * @param context
     *            see above
     * @param instance
     *            see above
     * @param param
     *            see above
     * @param width
     *            the width of the panel. The height is calculated using the
     *            GOLDEN_RATIO.
     */
    public LiveDataPanel(String context, String instance, String param,
            int width)
    {
        this(context, instance, param, width, (int) (width / GOLDEN_RATIO));
    }

    private void init()
    {
        ApplicationBusy.makeBusy(true);
        LiveDataService.Util.getInstance().getChartInfoAndInitialData(context,
                instance, param, new AsyncCallback<ChartInitData>()
                {
                    public void onFailure(Throwable caught)
                    {
                        displayError(caught.getMessage());
                        ApplicationBusy.makeBusy(false);
                    }

                    public void onSuccess(ChartInitData result)
                    {
                        sessionId = result.getSessionId();
                        initDisplay(result);
                        ApplicationBusy.makeBusy(false);
                    }
                });
    }

    private void initDisplay(ChartInitData data)
    {
        if (data.getChartData().getPlot() instanceof TimeSeriesPlotInfo)
        {
            initTimeSeries(data);
        }
        else
        {
            String message = "This type of plot is not supported";
            displayError(message);
        }
    }

    private void initTimeSeries(ChartInitData data)
    {
        TimeSeriesPlotInfo plot = (TimeSeriesPlotInfo) data.getChartData()
                .getPlot();

        if (data.getChartData().getRowsToDisplay() > 0)
        {
            initTable(plot, data.getChartData().getRowsToDisplay());
        }

        chartPanel = new TimeSeriesChartPanel(data.getChartData(), width,
                height);
        add(chartPanel);

        TimeSeriesPlotData[] cdata = (TimeSeriesPlotData[]) data
                .getInitialData().getData();
        initTimeSeriesData(plot, cdata, data.getChartData().getRowsToDisplay());

        if (data.getChartData().getRowsToDisplay() > 0)
        {

            table.setWidth(chartPanel.getOffsetWidth() + "px");
            add(table);
        }

        refresh = plot.getRefreshInterval();
        if (refresh > 0L)
        {
            initTimer();
        }
    }

    private void initTable(TimeSeriesPlotInfo plot, int numDataRows)
    {
        ColumnProperty[] cp = new ColumnProperty[plot.getSeries().length + 1];
        cp[0] = new ColumnProperty("Time", null, null);
        for (int i = 0; i < plot.getSeries().length; i++)
        {
            cp[i + 1] = new ColumnProperty(plot.getSeries()[i].getName(), null,
                    null);
        }

        SuperTableProperty prop = new SuperTableProperty();
        prop.setToolbarEnabled(false);
        prop.setCellSelectionEnabled(false);
        prop.setCommandBarEnabled(false);
        prop.setPagingEnabled(false);
        prop.setRecordsPerPage(0);
        prop.setRowSelectionEnabled(false);

        table = new SuperTable(cp, prop);
    }

    private void initTimer()
    {
        timer = new Timer()
        {
            public void run()
            {
                ApplicationBusy.makeBusy(true);
                LiveDataService.Util.getInstance().getNextData(sessionId,
                        new AsyncCallback<ChartData>()
                        {

                            public void onFailure(Throwable caught)
                            {
                                ApplicationBusy.makeBusy(false);
                                Window
                                        .alert("Error fetching data from the server: "
                                                + caught.getMessage());
                            }

                            public void onSuccess(ChartData data)
                            {
                                appendTimeSeriesData((TimeSeriesPlotData[]) data
                                        .getData());
                                ApplicationBusy.makeBusy(false);
                            }
                        });
            }
        };
        startRefresh();
    }

    private void appendTimeSeriesData(TimeSeriesPlotData[] cdata)
    {
        if (cdata.length == 0)
        {
            return;
        }

        for (int i = 0; i < cdata.length; i++)
        {
            chartPanel.addPoints(cdata[i].getTimestamp(), cdata[i].getValue());

            Widget[] widgets = null;

            if (table != null)
            {
                widgets = new Widget[cdata[i].getValue().length + 1];
                widgets[0] = new Label(cdata[i].getTimestamp().toString());
            }

            for (int j = 0; j < cdata[i].getValue().length; j++)
            {
                if (cdata[i].getValue()[j] == null)
                {
                    if (widgets != null)
                    {
                        widgets[j + 1] = new Label("-");
                    }
                }
                else
                {
                    Double v = cdata[i].getValue()[j];
                    if (widgets != null)
                    {
                        widgets[j + 1] = new Label(v.toString());
                    }
                }
            }

            if (widgets != null)
            {
                table.addRow(widgets);
                table.deleteRow(0);
            }
        }

        chartPanel.update();
    }

    private void initTimeSeriesData(TimeSeriesPlotInfo plot,
            TimeSeriesPlotData[] cdata, int numDataRows)
    {
        for (int i = 0; i < cdata.length; i++)
        {
            chartPanel.addPoints(cdata[i].getTimestamp(), cdata[i].getValue());
        }

        chartPanel.update();

        // if the data table is to be displayed
        // TODO - simplify the logic
        if (numDataRows > 0)
        {
            Date[] tstamp = new Date[numDataRows];
            Double[][] data = new Double[numDataRows][plot.getSeries().length];
            int start = 0;

            start = cdata.length - numDataRows < 0 ? 0 : cdata.length
                    - numDataRows;
            for (int i = start; i < cdata.length; i++)
            {
                tstamp[i - start] = cdata[i].getTimestamp();
            }

            for (int i = 0; i < plot.getSeries().length; i++)
            {
                for (int j = 0; j < cdata.length; j++)
                {
                    if (cdata[j].getValue()[i] == null)
                    {
                        continue;
                    }

                    if (j >= start)
                    {
                        data[i][j - start] = cdata[j].getValue()[i];
                    }
                }
            }

            for (int i = 0; i < tstamp.length && tstamp[i] != null; i++)
            {
                Widget[] widgets = new Widget[plot.getSeries().length + 1];

                widgets[0] = new Label(tstamp[i].toString());
                for (int j = 0; j < plot.getSeries().length; j++)
                {
                    widgets[j + 1] = new Label(data[i][j] == null ? "-"
                            : data[i][j].toString());
                }

                table.addRow(widgets);
            }
        }
    }

    private void displayError(String message)
    {
        MessageBar bar = new MessageBar();
        bar.setWidth("100%");
        add(bar);
        bar.setMessage(message, MessageBar.SEVERITY_ERROR);
    }

    /**
     * @return Returns the context.
     */
    public String getContext()
    {
        return context;
    }

    /**
     * @param context
     *            The context to set.
     */
    public void setContext(String context)
    {
        this.context = context;
    }

    /**
     * @return Returns the param.
     */
    public String getParam()
    {
        return param;
    }

    /**
     * @param param
     *            The param to set.
     */
    public void setParam(String param)
    {
        this.param = param;
    }

    /**
     * Stop refreshing the screen with new data.
     */
    public void stopRefresh()
    {
        if (timerStarted)
        {
            timer.cancel();
            timerStarted = false;
        }
    }

    /**
     * Start (restart) screen refresh.
     */
    public void startRefresh()
    {
        if (!timerStarted && refresh > 0L)
        {
            timer.scheduleRepeating((int) refresh);
            timerStarted = true;
        }
    }

    protected void onDetach()
    {
        super.onDetach();

        if (timer != null)
        {
            stopRefresh();
        }

        if (sessionId != -1)
        {
            LiveDataService.Util.getInstance().cancel(sessionId,
                    new AsyncCallback<Void>()
                    {
                        public void onFailure(Throwable caught)
                        {
                            Window
                                    .alert("An error occured while closing the live data panel");
                        }

                        public void onSuccess(Void result)
                        {
                        }
                    });
        }

    }

    protected void onAttach()
    {
        super.onAttach();

        if (timer != null)
        {
            startRefresh();
        }
    }

    /**
     * @return Returns the chartPanel.
     */
    public TimeSeriesChartPanel getChartPanel()
    {
        return chartPanel;
    }

    /**
     * @param chartPanel
     *            The chartPanel to set.
     */
    public void setChartPanel(TimeSeriesChartPanel chartPanel)
    {
        this.chartPanel = chartPanel;
    }
}
