/*
 * Created on Oct 13, 2007
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.client.utils;

import com.google.gwt.core.client.JavaScriptException;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.InvocationException;

/**
 * This abstract class can be used in place of GWT AsyncCallback to
 * automatically handle server communication failure including authentication
 * request from the server. An authentication request is sent by the servlet
 * container to the browser in response to a HTTP request when the HTTP session
 * has timed out because of inactivity and the URL being accessed has an
 * authentication constraint. An RPC call is a HTTP request and therefore, is
 * given the same treatment. GWT does not handle this very well. The RPC call
 * fails with an exception and the application has to handle this. This class
 * takes care of the problem by handling the exception transparently. When an
 * RPC invocation fails, GWT throws an InvocationException. This class assumes
 * that there is a either a session timeout or the server is down. This class
 * displays an alert message and redirects to the login page.
 * <p>
 * For this to work, the static setLoginURL() method must be called prior to
 * using this class to set the login page.
 * <p>
 * The abstract methods are invoked when the RPC response is received from the
 * server.
 * <p>
 * 
 * @author Amit Chatterjee
 * 
 */
public abstract class AsyncCallback<T> implements
        com.google.gwt.user.client.rpc.AsyncCallback<T>
{
    private static String loginURL = null;

    /*
     * @see
     * com.google.gwt.user.client.rpc.AsyncCallback#onFailure(java.lang.Throwable
     * )
     */
    public final void onFailure(Throwable caught)
    {
        try
        {
            if (caught instanceof InvocationException)
            {
                String message = caught.getMessage();
                if (message != null & message.length() > 0)
                {
                    message = message + ".";
                }
                else
                {
                    message = "There was an error communicating with the server. ";
                }
                Window
                        .alert(message
                                + "Please relogin and try again. "
                                + "If the problem persists, please contact your service provider");
                if (loginURL != null)
                {
                    redirect(loginURL);
                }
                return;
            }

            onException(caught);
        }
        catch (JavaScriptException e)
        {
            Window.alert("An error occured while processing the request: "
                    + e.toString() + ": " + e.getMessage());
        }
    }

    /*
     * @see
     * com.google.gwt.user.client.rpc.AsyncCallback#onSuccess(java.lang.Object)
     */
    public abstract void onSuccess(T result);

    /**
     * This method is invoked when the GWTServlet throws an exception. Note that
     * InvocationException thrown because of session-timeouts are automatically
     * handled.
     * 
     * @param caught
     */
    public abstract void onException(Throwable caught);

    private static native void redirect(String url) /*-{
                    $wnd.location = url;
                    }-*/;

    /**
     * @return Returns the loginURL.
     */
    public static String getLoginURL()
    {
        return loginURL;
    }

    /**
     * Set the login URL. When the login form is received in response to a RPC
     * request, this object redirects it to the page. Typically the page is set
     * to GWT.getModuleBaseURL() + "appname.html" where the appname.html is the
     * starter HTML for the GWT application.
     * 
     * @param loginURL
     *            The loginURL to set.
     */
    public static void setLoginURL(String loginURL)
    {
        AsyncCallback.loginURL = loginURL;
    }
}
