/*
 * Created on Mar 27, 2008
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *  http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.server;

import java.security.Principal;
import java.util.Enumeration;
import java.util.HashMap;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.cafesip.gwtcomp.client.common.ChartData;
import org.cafesip.gwtcomp.client.common.ChartInitData;
import org.cafesip.gwtcomp.client.common.LiveDataService;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * A GWT servlet for handling the server-side RPC functions needed by the
 * {@link org.cafesip.gwtcomp.client.ui.charting.LiveDataPanel} to display and
 * refresh data dynamically. This class must be added to the web.xml of your
 * application as explained in the documentation for the LiveDataPanel.
 * 
 * @author Amit Chatterjee
 * 
 */
public class LiveDataServiceImpl extends RemoteServiceServlet implements
        LiveDataService
{
    private static final String LIVE_DATA_SOURCE_ACTION_MAP = "liveDataSourceActionMap";

    private static final String DATA_SOURCE_CLASS_INIT_PARAM = "data-source-class";

    private static final String LIVE_DATA_SOURCE_DEFAULT_CLASS = "liveDataSourceDefaultClass";

    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = -111985756264744546L;

    /*
     * @see javax.servlet.GenericServlet#init(javax.servlet.ServletConfig)
     */
    @Override
    public void init(ServletConfig config) throws ServletException
    {
        super.init(config);

        boolean def_found = false;
        Enumeration<String> en = config.getInitParameterNames();
        HashMap<String, LiveDataSource> actions = new HashMap<String, LiveDataSource>();
        try
        {
            while (en.hasMoreElements())
            {
                boolean def = false;
                String param = en.nextElement();
                if (param.startsWith(DATA_SOURCE_CLASS_INIT_PARAM))
                {
                    String context = null;
                    String clazz = config.getInitParameter(param);
                    ;
                    if (param.length() == DATA_SOURCE_CLASS_INIT_PARAM.length())
                    {
                        // default class
                        def_found = def = true;
                    }
                    else
                    {
                        context = param
                                .substring(DATA_SOURCE_CLASS_INIT_PARAM
                                        .length());
                        if (!context.startsWith("-"))
                        {
                            continue;
                        }

                        context = context.substring(1);
                        if (context.length() == 0)
                        {
                            throw new ServletException(
                                    "A "
                                            + DATA_SOURCE_CLASS_INIT_PARAM
                                            + " init parameter has been specified with no context ");
                        }
                    }

                    Class<?> c = Class.forName(clazz);
                    if (!LiveDataSource.class.isAssignableFrom(c))
                    {
                        throw new ServletException(
                                "This class name specified by the "
                                        + DATA_SOURCE_CLASS_INIT_PARAM
                                        + " init parameter does not implement the "
                                        + LiveDataSource.class.getName()
                                        + " interface (" + clazz + ")");
                    }

                    if (def)
                    {
                        config.getServletContext()
                                .setAttribute(LIVE_DATA_SOURCE_DEFAULT_CLASS,
                                        c.newInstance());
                    }
                    else
                    {
                        actions.put(context, (LiveDataSource) c
                                .newInstance());
                    }
                }
            }

            config.getServletContext().setAttribute(
                    LIVE_DATA_SOURCE_ACTION_MAP, actions);

            if (!def_found)
            {
                config.getServletContext().setAttribute(
                        LIVE_DATA_SOURCE_DEFAULT_CLASS,
                        new SimpleLiveDataSource());
            }
        }
        catch (Exception e)
        {
            throw new ServletException(e);
        }
    }

    /**
     * 
     */
    public ChartInitData getChartInfoAndInitialData(String context,
            String instance, String param)
    {
        HttpServletRequest request = getThreadLocalRequest();
        Principal user = request.getUserPrincipal();
        HttpSession session = request.getSession(true);

        HashMap<String, LiveDataSource> map = (HashMap<String, LiveDataSource>) getServletContext()
                .getAttribute(LIVE_DATA_SOURCE_ACTION_MAP);

        LiveDataSource source = map.get(context);
        if (source == null)
        {
            source = (LiveDataSource) getServletContext().getAttribute(
                    LIVE_DATA_SOURCE_DEFAULT_CLASS);
        }
        
        LiveDataSessionInfo sd = new LiveDataSessionInfo(context, 
                instance, param, null, 
                null, 0);
        
        // invoke the listener method
        ChartInitData chart = source.getChartInfoAndInitialData(user, session, sd);
        
        // store session information
        sd.setChartInfo(chart.getChartData());
        sd.setLastCollection(chart.getInitialData().getCollectionTime());
        sd.setDeliveryCount(1);
        int session_id = LiveDataSourceSession.addSession(sd, session);
        chart.setSessionId(session_id);
        return chart;
    }

    /**
     * 
     */
    public ChartData getNextData(int sessionId)
    {
        HttpServletRequest request = getThreadLocalRequest();
        Principal user = request.getUserPrincipal();
        HttpSession session = request.getSession();

        LiveDataSessionInfo sd = LiveDataSourceSession.getSession(sessionId, session);
        if (sd == null)
        {
            throw new RuntimeException("Received a request to fetch " +
            		"Live data source data for a session that does not exist");
        }
        
        HashMap<String, LiveDataSource> map = (HashMap<String, LiveDataSource>) getServletContext()
                .getAttribute(LIVE_DATA_SOURCE_ACTION_MAP);

        LiveDataSource source = map.get(sd.getContext());
        if (source == null)
        {
            source = (LiveDataSource) getServletContext().getAttribute(
                    LIVE_DATA_SOURCE_DEFAULT_CLASS);
        }

        try
        {        
            ChartData cd = source.getNextData(user, session, sd);
            sd.setDeliveryCount(sd.getDeliveryCount() + 1);
            return cd;
        }
        catch (Exception e)
        {
            LiveDataSourceSession.removeSession(sessionId, session);
            throw new RuntimeException(e);
        }
    }

    /*
     * @see org.cafesip.gwtcomp.client.common.LiveDataService#cancel(int)
     */
    public void cancel(int sessionId)
    {
        HttpServletRequest request = getThreadLocalRequest();
        Principal user = request.getUserPrincipal();
        HttpSession session = request.getSession();

        LiveDataSessionInfo sd = LiveDataSourceSession.getSession(sessionId, session);
        if (sd == null)
        {
            throw new RuntimeException("Received a request to fetch " +
                    "Live data source data for a session that does not exist");
        }
        
        HashMap<String, LiveDataSource> map = (HashMap<String, LiveDataSource>) getServletContext()
        .getAttribute(LIVE_DATA_SOURCE_ACTION_MAP);
        LiveDataSource source = map.get(sd.getContext());
        if (source == null)
        {
            source = (LiveDataSource) getServletContext().getAttribute(
                    LIVE_DATA_SOURCE_DEFAULT_CLASS);
        }

        source.close(user, session, sd);        
        LiveDataSourceSession.removeSession(sessionId, session);        
    }
}
