/*
 * Created on Mar 27, 2008
 * 
 * Copyright 2005 CafeSip.org 
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *	http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 */
package org.cafesip.gwtcomp.server;

import java.security.Principal;
import java.util.Date;
import java.util.Random;

import javax.servlet.http.HttpSession;

import org.cafesip.gwtcomp.client.common.ChartData;
import org.cafesip.gwtcomp.client.common.ChartInfo;
import org.cafesip.gwtcomp.client.common.ChartInitData;
import org.cafesip.gwtcomp.client.common.SeriesInfo;
import org.cafesip.gwtcomp.client.common.TimeSeriesPlotData;
import org.cafesip.gwtcomp.client.common.TimeSeriesPlotInfo;

/**
 * This class implements a simple live data source that can be used for
 * demonstration purposes.
 * 
 * @author Amit Chatterjee
 * 
 */
public class SimpleLiveDataSource implements LiveDataSource
{
    /**
     * Enumerates the data-generation strategy. Based on what value if specified
     * by the GWT application, the data is generated accordingly.
     * 
     */
    public enum LiveDataStrategy {
        /**
         * Generate data using random numbers.
         */
        RANDOM
    }

    private Random rand = new Random(new Date().getTime());

    private LiveDataStrategy strategy = LiveDataStrategy.RANDOM;

    private int numSeries = 5;

    private double rangeTop = 100.0;

    private double rangeBottom = 0.0;

    private String[] seriesNames = { "Series 1", "Series 2", "Series 3",
            "Series 4", "Series 5" };

    private String title = "Simple Series";

    private long refreshInterval = 30 * 1000L;

    /**
     * 
     * A constructor for this class.
     * 
     */
    public SimpleLiveDataSource()
    {
    }
    
    public ChartInitData getChartInfoAndInitialData(Principal user,
            HttpSession session,  LiveDataSessionInfo sessionData)
    {
        SeriesInfo[] serie = new SeriesInfo[numSeries];

        for (int i = 0; i < numSeries; i++)
        {
            serie[i] = new SeriesInfo(seriesNames[i]);
        }

        TimeSeriesPlotInfo plot = new TimeSeriesPlotInfo(serie, rangeBottom,
                rangeTop, refreshInterval);

        ChartInfo cinfo = new ChartInfo(title, true, plot, 5);

        TimeSeriesPlotData[] pdata = initInitialData(
                (int) (5 * refreshInterval), (int) refreshInterval);

        ChartData cd = new ChartData(new Date(), pdata);
        ChartInitData cdata = new ChartInitData(cinfo, cd);

        return cdata;

    }

    public ChartData getNextData(Principal user, HttpSession session,
            LiveDataSessionInfo sessionData)
    {
        TimeSeriesPlotData[] pdata = new TimeSeriesPlotData[1];

        long dt = new Date().getTime();

        Double[] v = new Double[numSeries];
        for (int j = 0; j < v.length; j++)
        {
            if (strategy == LiveDataStrategy.RANDOM)
            {
                v[j] = new Double(genRandom());
            }
        }
        pdata[0] = new TimeSeriesPlotData(new Date(dt), v);
        ChartData cd = new ChartData(new Date(), pdata);
        return cd;
    }

    private TimeSeriesPlotData[] initInitialData(int period, int interval)
    {
        int times = period / interval;

        TimeSeriesPlotData[] pdata = new TimeSeriesPlotData[times];
        long dt = new Date().getTime();
        for (int i = 0; i < times; i++)
        {
            Double[] v = new Double[numSeries];
            for (int j = 0; j < v.length; j++)
            {
                if (strategy == LiveDataStrategy.RANDOM)
                {
                    v[j] = new Double(genRandom());
                }
            }
            pdata[i] = new TimeSeriesPlotData(new Date(dt
                    - (interval * (times - (i + 1)))), v);
        }

        return pdata;
    }

    private Double genRandom()
    {
        return new Double(rand.nextInt(100));
    }

    /**
     * @return Returns the strategy.
     */
    public LiveDataStrategy getStrategy()
    {
        return strategy;
    }

    /**
     * @param strategy
     *            The strategy to set.
     */
    public void setStrategy(LiveDataStrategy strategy)
    {
        this.strategy = strategy;
    }

    /**
     * @return Returns the numSeries.
     */
    public int getNumSeries()
    {
        return numSeries;
    }

    /**
     * @param numSeries
     *            The numSeries to set.
     */
    public void setNumSeries(int numSeries)
    {
        this.numSeries = numSeries;
    }

    /**
     * @return Returns the rangeTop.
     */
    public double getRangeTop()
    {
        return rangeTop;
    }

    /**
     * @param rangeTop
     *            The rangeTop to set.
     */
    public void setRangeTop(double rangeTop)
    {
        this.rangeTop = rangeTop;
    }

    /**
     * @return Returns the rangeBottom.
     */
    public double getRangeBottom()
    {
        return rangeBottom;
    }

    /**
     * @param rangeBottom
     *            The rangeBottom to set.
     */
    public void setBottomRange(double rangeBottom)
    {
        this.rangeBottom = rangeBottom;
    }

    /**
     * @return Returns the seriesNames.
     */
    public String[] getSeriesNames()
    {
        return seriesNames;
    }

    /**
     * @param seriesNames
     *            The seriesNames to set.
     */
    public void setSeriesNames(String[] seriesNames)
    {
        this.seriesNames = seriesNames;
    }

    /**
     * @return Returns the title.
     */
    public String getTitle()
    {
        return title;
    }

    /**
     * @param title
     *            The title to set.
     */
    public void setTitle(String title)
    {
        this.title = title;
    }

    /**
     * @return Returns the refreshInterval.
     */
    public long getRefreshInterval()
    {
        return refreshInterval;
    }

    /**
     * @param refreshInterval
     *            The refreshInterval to set.
     */
    public void setRefreshInterval(long refreshInterval)
    {
        this.refreshInterval = refreshInterval;
    }

    public void close(Principal user, HttpSession session,
            LiveDataSessionInfo sessionData)
    {
    }

}
